@@ -23,30 +23,33 @@ class ParametersSerializationTest < ActiveSupport::TestCase
 
   test "yaml deserialization" do
     params = ActionController::Parameters.new(key: :value)
-    roundtripped = YAML.load(YAML.dump(params))
+    payload = YAML.dump(params)
+    roundtripped = YAML.respond_to?(:unsafe_load) ? YAML.unsafe_load(payload) : YAML.load(payload)
 
     assert_equal params, roundtripped
     assert_not_predicate roundtripped, :permitted?
   end
 
   test "yaml backwardscompatible with psych 2.0.8 format" do
-    params = YAML.load <<~end_of_yaml
+    payload = <<~end_of_yaml
       --- !ruby/hash:ActionController::Parameters
       key: :value
     end_of_yaml
+    params = YAML.respond_to?(:unsafe_load) ? YAML.unsafe_load(payload) : YAML.load(payload)
 
     assert_equal :value, params[:key]
     assert_not_predicate params, :permitted?
   end
 
   test "yaml backwardscompatible with psych 2.0.9+ format" do
-    params = YAML.load(<<~end_of_yaml)
+    payload = <<~end_of_yaml
       --- !ruby/hash-with-ivars:ActionController::Parameters
       elements:
         key: :value
       ivars:
         :@permitted: false
     end_of_yaml
+    params = YAML.respond_to?(:unsafe_load) ? YAML.unsafe_load(payload) : YAML.load(payload)
 
     assert_equal :value, params[:key]
     assert_not_predicate params, :permitted?
