@@ -66,6 +66,12 @@ module Namespaced
   end
 end
 
+class InheritedRenderTestController < ImplicitRenderTestController
+  def hello_world
+    fresh_when(etag: "abc")
+  end
+end
+
 class TestController < ActionController::Base
   protect_from_forgery
 
@@ -223,6 +229,10 @@ class TestController < ActionController::Base
     head :ok, content_type: "image/png"
   end
 
+  def head_ok_with_string_key_content_type
+    head :ok, "Content-Type" => "application/pdf"
+  end
+
   def head_with_location_header
     head :ok, location: "/foo"
   end
@@ -339,10 +349,8 @@ class ExpiresInRenderTest < ActionController::TestCase
 
   def test_dynamic_render_with_file
     assert File.exist?(File.expand_path("../../test/abstract_unit.rb", __dir__))
-    assert_deprecated do
-      assert_raises ActionView::MissingTemplate do
-        get :dynamic_render_with_file, params: { id: '../\\../test/abstract_unit.rb' }
-      end
+    assert_raises ArgumentError do
+      get :dynamic_render_with_file, params: { id: '../\\../test/abstract_unit.rb' }
     end
   end
 
@@ -361,16 +369,16 @@ class ExpiresInRenderTest < ActionController::TestCase
   def test_dynamic_render
     assert File.exist?(File.expand_path("../../test/abstract_unit.rb", __dir__))
     assert_raises ActionView::MissingTemplate do
-      get :dynamic_render, params: { id: '../\\../test/abstract_unit.rb' }
+      assert_deprecated do
+        get :dynamic_render, params: { id: '../\\../test/abstract_unit.rb' }
+      end
     end
   end
 
   def test_permitted_dynamic_render_file_hash
     assert File.exist?(File.expand_path("../../test/abstract_unit.rb", __dir__))
-    assert_deprecated do
-      assert_raises ActionView::MissingTemplate do
-        get :dynamic_render_permit, params: { id: { file: '../\\../test/abstract_unit.rb' } }
-      end
+    assert_raises ArgumentError do
+      get :dynamic_render_permit, params: { id: { file: '../\\../test/abstract_unit.rb' } }
     end
   end
 
@@ -675,6 +683,28 @@ class NamespacedEtagRenderTest < ActionController::TestCase
   end
 end
 
+class InheritedEtagRenderTest < ActionController::TestCase
+  tests InheritedRenderTestController
+  include TemplateModificationHelper
+
+  def test_etag_reflects_template_digest
+    get :hello_world
+    assert_response :ok
+    assert_not_nil etag = @response.etag
+
+    request.if_none_match = etag
+    get :hello_world
+    assert_response :not_modified
+
+    modify_template("implicit_render_test/hello_world") do
+      request.if_none_match = etag
+      get :hello_world
+      assert_response :ok
+      assert_not_equal etag, @response.etag
+    end
+  end
+end
+
 class MetalRenderTest < ActionController::TestCase
   tests MetalTestController
 
@@ -757,6 +787,11 @@ class HeadRenderTest < ActionController::TestCase
     assert_response :ok
   end
 
+  def test_head_respect_string_content_type
+    get :head_ok_with_string_key_content_type
+    assert_equal "application/pdf", @response.header["Content-Type"]
+  end
+
   def test_head_with_location_header
     get :head_with_location_header
     assert_predicate @response.body, :blank?
@@ -858,6 +893,38 @@ class HeadRenderTest < ActionController::TestCase
   end
 end
 
+class LiveTestController < ActionController::Base
+  include ActionController::Live
+
+  def test_action
+    head :ok
+  end
+end
+
+class LiveHeadRenderTest < ActionController::TestCase
+  tests LiveTestController
+
+  def setup
+    super
+
+    def @controller.new_controller_thread
+      Thread.new { yield }
+    end
+
+    def @controller.response_body=(body)
+      super
+      sleep 0.1
+    end
+  end
+
+  def test_live_head_ok
+    get :test_action, format: "json"
+
+    @response.stream.on_error { flunk "action should not raise any errors" }
+    sleep 0.2
+  end
+end
+
 class HttpCacheForeverTest < ActionController::TestCase
   class HttpCacheForeverController < ActionController::Base
     def cache_me_forever
