@@ -16,6 +16,7 @@ module TestUrlGeneration
 
     Routes.draw do
       get "/foo", to: "my_route_generating#index", as: :foo
+      get "(/optional/:optional_id)/baz", to: "my_route_generating#index", as: :baz
 
       resources :bars
 
@@ -55,6 +56,17 @@ module TestUrlGeneration
       assert_equal "http://www.example.com/foo", foo_url(protocol: "http")
     end
 
+    test "respects secure_protocol configuration when protocol not present" do
+      old_secure_protocol = ActionDispatch::Http::URL.secure_protocol
+
+      begin
+        ActionDispatch::Http::URL.secure_protocol = true
+        assert_equal "https://www.example.com/foo", foo_url(protocol: nil)
+      ensure
+        ActionDispatch::Http::URL.secure_protocol = old_secure_protocol
+      end
+    end
+
     test "extracting protocol from host when protocol not present" do
       assert_equal "httpz://www.example.com/foo", foo_url(host: "httpz://www.example.com", protocol: nil)
     end
@@ -116,6 +128,34 @@ module TestUrlGeneration
       assert_equal "http://example.com/foo", foo_url(subdomain: "")
     end
 
+    test "keep optional path parameter when given" do
+      assert_equal "http://www.example.com/optional/123/baz", baz_url(optional_id: 123)
+    end
+
+    test "keep optional path parameter when true" do
+      assert_equal "http://www.example.com/optional/true/baz", baz_url(optional_id: true)
+    end
+
+    test "omit optional path parameter when false" do
+      assert_equal "http://www.example.com/optional/false/baz", baz_url(optional_id: false)
+    end
+
+    test "omit optional path parameter when blank" do
+      assert_equal "http://www.example.com/baz", baz_url(optional_id: "")
+    end
+
+    test "keep positional path parameter when true" do
+      assert_equal "http://www.example.com/optional/true/baz", baz_url(true)
+    end
+
+    test "omit positional path parameter when false" do
+      assert_equal "http://www.example.com/optional/false/baz", baz_url(false)
+    end
+
+    test "omit positional path parameter when blank" do
+      assert_equal "http://www.example.com/baz", baz_url("")
+    end
+
     test "generating URLs with trailing slashes" do
       assert_equal "/bars.json", bars_path(
         trailing_slash: true,
