@@ -3,8 +3,8 @@
 require "abstract_unit"
 
 class TextTest < ActiveSupport::TestCase
-  test "formats always return :text" do
-    assert_equal [:text], ActionView::Template::Text.new("").formats
+  test "format always return :text" do
+    assert_equal :text, ActionView::Template::Text.new("").format
   end
 
   test "identifier should return 'text template'" do
