@@ -35,20 +35,20 @@ class RailtieTest < ActiveModel::TestCase
   test "i18n customize full message defaults to false" do
     @app.initialize!
 
-    assert_equal false, ActiveModel::Errors.i18n_customize_full_message
+    assert_equal false, ActiveModel::Error.i18n_customize_full_message
   end
 
   test "i18n customize full message can be disabled" do
     @app.config.active_model.i18n_customize_full_message = false
     @app.initialize!
 
-    assert_equal false, ActiveModel::Errors.i18n_customize_full_message
+    assert_equal false, ActiveModel::Error.i18n_customize_full_message
   end
 
   test "i18n customize full message can be enabled" do
     @app.config.active_model.i18n_customize_full_message = true
     @app.initialize!
 
-    assert_equal true, ActiveModel::Errors.i18n_customize_full_message
+    assert_equal true, ActiveModel::Error.i18n_customize_full_message
   end
 end
