@@ -16,16 +16,20 @@ module ActiveModel
 
       test "a block can be registered" do
         registry = Type::Registry.new
-        registry.register(:foo) do |*args|
-          [*args, "block for foo"]
+        registry.register(:foo) do |type, *args|
+          [type, args, "block for foo"]
         end
-        registry.register(:bar) do |*args|
-          [*args, "block for bar"]
+        registry.register(:bar) do |type, *args|
+          [type, args, "block for bar"]
+        end
+        registry.register(:baz) do |type, **kwargs|
+          [type, kwargs, "block for baz"]
         end
 
-        assert_equal [:foo, 1, "block for foo"], registry.lookup(:foo, 1)
-        assert_equal [:foo, 2, "block for foo"], registry.lookup(:foo, 2)
-        assert_equal [:bar, 1, 2, 3, "block for bar"], registry.lookup(:bar, 1, 2, 3)
+        assert_equal [:foo, [1], "block for foo"], registry.lookup(:foo, 1)
+        assert_equal [:foo, [2], "block for foo"], registry.lookup(:foo, 2)
+        assert_equal [:bar, [1, 2, 3], "block for bar"], registry.lookup(:bar, 1, 2, 3)
+        assert_equal [:baz, { kw: 1 }, "block for baz"], registry.lookup(:baz, kw: 1)
       end
 
       test "a reasonable error is given when no type is found" do
