@@ -19,7 +19,7 @@ class Contact
   end
 
   def initialize(options = {})
-    options.each { |name, value| send("#{name}=", value) }
+    options.each { |name, value| public_send("#{name}=", value) }
   end
 
   def pseudonyms
