@@ -12,7 +12,7 @@ require "arel/math"
 require "arel/alias_predication"
 require "arel/order_predications"
 require "arel/table"
-require "arel/attributes"
+require "arel/attributes/attribute"
 
 require "arel/visitors"
 require "arel/collectors/sql_string"
@@ -43,20 +43,12 @@ module Arel
   end
 
   def self.arel_node?(value) # :nodoc:
-    value.is_a?(Arel::Node) || value.is_a?(Arel::Attribute) || value.is_a?(Arel::Nodes::SqlLiteral)
+    value.is_a?(Arel::Nodes::Node) || value.is_a?(Arel::Attribute) || value.is_a?(Arel::Nodes::SqlLiteral)
   end
 
-  def self.fetch_attribute(value) # :nodoc:
-    case value
-    when Arel::Nodes::Between, Arel::Nodes::In, Arel::Nodes::NotIn, Arel::Nodes::Equality, Arel::Nodes::NotEqual, Arel::Nodes::LessThan, Arel::Nodes::LessThanOrEqual, Arel::Nodes::GreaterThan, Arel::Nodes::GreaterThanOrEqual
-      if value.left.is_a?(Arel::Attributes::Attribute)
-        yield value.left
-      elsif value.right.is_a?(Arel::Attributes::Attribute)
-        yield value.right
-      end
+  def self.fetch_attribute(value, &block) # :nodoc:
+    unless String === value
+      value.fetch_attribute(&block)
     end
   end
-
-  ## Convenience Alias
-  Node = Arel::Nodes::Node # :nodoc:
 end
