@@ -3,6 +3,7 @@
 require "cases/helper"
 require "models/developer"
 require "models/computer"
+require "active_support/core_ext/symbol/starts_ends_with"
 
 class CallbackDeveloper < ActiveRecord::Base
   self.table_name = "developers"
@@ -21,7 +22,7 @@ class CallbackDeveloper < ActiveRecord::Base
 
     def callback_object(callback_method)
       klass = Class.new
-      klass.send(:define_method, callback_method) do |model|
+      klass.define_method(callback_method) do |model|
         model.history << [callback_method, :object]
       end
       klass.new
@@ -29,7 +30,7 @@ class CallbackDeveloper < ActiveRecord::Base
   end
 
   ActiveRecord::Callbacks::CALLBACKS.each do |callback_method|
-    next if callback_method.to_s.start_with?("around_")
+    next if callback_method.start_with?("around_")
     define_callback_method(callback_method)
     send(callback_method, callback_proc(callback_method))
     send(callback_method, callback_object(callback_method))
@@ -385,9 +386,9 @@ class CallbacksTest < ActiveRecord::TestCase
   end
 
   def assert_save_callbacks_not_called(someone)
-    assert !someone.after_save_called
-    assert !someone.after_create_called
-    assert !someone.after_update_called
+    assert_not someone.after_save_called
+    assert_not someone.after_create_called
+    assert_not someone.after_update_called
   end
   private :assert_save_callbacks_not_called
 
@@ -395,27 +396,27 @@ class CallbacksTest < ActiveRecord::TestCase
     someone = CallbackHaltedDeveloper.new
     someone.cancel_before_create = true
     assert_predicate someone, :valid?
-    assert !someone.save
+    assert_not someone.save
     assert_save_callbacks_not_called(someone)
   end
 
   def test_before_save_throwing_abort
     david = DeveloperWithCanceledCallbacks.find(1)
     assert_predicate david, :valid?
-    assert !david.save
+    assert_not david.save
     exc = assert_raise(ActiveRecord::RecordNotSaved) { david.save! }
     assert_equal david, exc.record
 
     david = DeveloperWithCanceledCallbacks.find(1)
     david.salary = 10_000_000
     assert_not_predicate david, :valid?
-    assert !david.save
+    assert_not david.save
     assert_raise(ActiveRecord::RecordInvalid) { david.save! }
 
     someone = CallbackHaltedDeveloper.find(1)
     someone.cancel_before_save = true
     assert_predicate someone, :valid?
-    assert !someone.save
+    assert_not someone.save
     assert_save_callbacks_not_called(someone)
   end
 
@@ -423,22 +424,22 @@ class CallbacksTest < ActiveRecord::TestCase
     someone = CallbackHaltedDeveloper.find(1)
     someone.cancel_before_update = true
     assert_predicate someone, :valid?
-    assert !someone.save
+    assert_not someone.save
     assert_save_callbacks_not_called(someone)
   end
 
   def test_before_destroy_throwing_abort
     david = DeveloperWithCanceledCallbacks.find(1)
-    assert !david.destroy
+    assert_not david.destroy
     exc = assert_raise(ActiveRecord::RecordNotDestroyed) { david.destroy! }
     assert_equal david, exc.record
     assert_not_nil ImmutableDeveloper.find_by_id(1)
 
     someone = CallbackHaltedDeveloper.find(1)
     someone.cancel_before_destroy = true
-    assert !someone.destroy
+    assert_not someone.destroy
     assert_raise(ActiveRecord::RecordNotDestroyed) { someone.destroy! }
-    assert !someone.after_destroy_called
+    assert_not someone.after_destroy_called
   end
 
   def test_callback_throwing_abort
@@ -458,22 +459,45 @@ class CallbacksTest < ActiveRecord::TestCase
       [ :before_validation, :object ],
       [ :before_validation, :block  ],
       [ :before_validation, :throwing_abort ],
-      [ :after_rollback,    :block  ],
-      [ :after_rollback,    :object ],
-      [ :after_rollback,    :proc   ],
-      [ :after_rollback,    :method ],
     ], david.history
   end
 
   def test_inheritance_of_callbacks
     parent = ParentDeveloper.new
-    assert !parent.after_save_called
+    assert_not parent.after_save_called
     parent.save
     assert parent.after_save_called
 
     child = ChildDeveloper.new
-    assert !child.after_save_called
+    assert_not child.after_save_called
     child.save
     assert child.after_save_called
   end
+
+  def test_before_save_doesnt_allow_on_option
+    exception = assert_raises ArgumentError do
+      Class.new(ActiveRecord::Base) do
+        before_save(on: :create) { }
+      end
+    end
+    assert_equal "Unknown key: :on. Valid keys are: :if, :unless, :prepend", exception.message
+  end
+
+  def test_around_save_doesnt_allow_on_option
+    exception = assert_raises ArgumentError do
+      Class.new(ActiveRecord::Base) do
+        around_save(on: :create) { }
+      end
+    end
+    assert_equal "Unknown key: :on. Valid keys are: :if, :unless, :prepend", exception.message
+  end
+
+  def test_after_save_doesnt_allow_on_option
+    exception = assert_raises ArgumentError do
+      Class.new(ActiveRecord::Base) do
+        after_save(on: :create) { }
+      end
+    end
+    assert_equal "Unknown key: :on. Valid keys are: :if, :unless, :prepend", exception.message
+  end
 end
