@@ -11,28 +11,31 @@ module ActiveRecord
       fixtures :people
 
       def setup
-        @handlers = { writing: ConnectionHandler.new, reading: ConnectionHandler.new }
-        @rw_handler = @handlers[:writing]
-        @ro_handler = @handlers[:reading]
-        @spec_name = "primary"
-        @rw_pool = @handlers[:writing].establish_connection(ActiveRecord::Base.configurations["arunit"])
-        @ro_pool = @handlers[:reading].establish_connection(ActiveRecord::Base.configurations["arunit"])
+        @handler = ConnectionHandler.new
+        @owner_name = "ActiveRecord::Base"
+        db_config = ActiveRecord::Base.configurations.configs_for(env_name: "arunit", name: "primary")
+        @rw_pool = @handler.establish_connection(db_config)
+        @ro_pool = @handler.establish_connection(db_config, role: :reading)
       end
 
       def teardown
-        ActiveRecord::Base.connection_handlers = { writing: ActiveRecord::Base.default_connection_handler }
+        clean_up_connection_handler
       end
 
-      class MultiConnectionTestModel < ActiveRecord::Base
+      class SecondaryBase < ActiveRecord::Base
+        self.abstract_class = true
       end
 
-      def test_multiple_connection_handlers_works_in_a_threaded_environment
+      class MultiConnectionTestModel < SecondaryBase
+      end
+
+      def test_multiple_connections_works_in_a_threaded_environment
         tf_writing = Tempfile.open "test_writing"
         tf_reading = Tempfile.open "test_reading"
 
         # We need to use a role for reading not named reading, otherwise we'll prevent writes
         # and won't be able to write to the second connection.
-        MultiConnectionTestModel.connects_to database: { writing: { database: tf_writing.path, adapter: "sqlite3" }, secondary: { database: tf_reading.path, adapter: "sqlite3" } }
+        SecondaryBase.connects_to database: { writing: { database: tf_writing.path, adapter: "sqlite3" }, secondary: { database: tf_reading.path, adapter: "sqlite3" } }
 
         MultiConnectionTestModel.connection.execute("CREATE TABLE `multi_connection_test_models` (connection_role VARCHAR (255))")
         MultiConnectionTestModel.connection.execute("INSERT INTO multi_connection_test_models VALUES ('writing')")
@@ -69,10 +72,10 @@ module ActiveRecord
         tf_writing.unlink
       end
 
-      def test_loading_relations_with_multi_db_connection_handlers
+      def test_loading_relations_with_multi_db_connections
         # We need to use a role for reading not named reading, otherwise we'll prevent writes
         # and won't be able to write to the second connection.
-        MultiConnectionTestModel.connects_to database: { writing: { database: ":memory:", adapter: "sqlite3" }, secondary: { database: ":memory:", adapter: "sqlite3" } }
+        SecondaryBase.connects_to database: { writing: { database: ":memory:", adapter: "sqlite3" }, secondary: { database: ":memory:", adapter: "sqlite3" } }
 
         relation = ActiveRecord::Base.connected_to(role: :secondary) do
           MultiConnectionTestModel.connection.execute("CREATE TABLE `multi_connection_test_models` (connection_role VARCHAR (255))")
@@ -89,19 +92,21 @@ module ActiveRecord
 
           config = {
             "default_env" => {
-              "readonly" => { "adapter" => "sqlite3", "database" => "db/readonly.sqlite3" },
-              "primary"  => { "adapter" => "sqlite3", "database" => "db/primary.sqlite3" }
+              "readonly" => { "adapter" => "sqlite3", "database" => "test/db/readonly.sqlite3", "replica" => true },
+              "default"  => { "adapter" => "sqlite3", "database" => "test/db/primary.sqlite3" }
             }
           }
           @prev_configs, ActiveRecord::Base.configurations = ActiveRecord::Base.configurations, config
 
-          ActiveRecord::Base.connects_to(database: { writing: :primary, reading: :readonly })
+          ActiveRecord::Base.connects_to(database: { writing: :default, reading: :readonly })
 
-          assert_not_nil pool = ActiveRecord::Base.connection_handlers[:writing].retrieve_connection_pool("primary")
-          assert_equal "db/primary.sqlite3", pool.spec.config[:database]
+          assert_not_nil pool = ActiveRecord::Base.connection_handler.retrieve_connection_pool("ActiveRecord::Base")
+          assert_equal "test/db/primary.sqlite3", pool.db_config.database
+          assert_equal "default", pool.db_config.name
 
-          assert_not_nil pool = ActiveRecord::Base.connection_handlers[:reading].retrieve_connection_pool("primary")
-          assert_equal "db/readonly.sqlite3", pool.spec.config[:database]
+          assert_not_nil pool = ActiveRecord::Base.connection_handler.retrieve_connection_pool("ActiveRecord::Base", role: :reading)
+          assert_equal "test/db/readonly.sqlite3", pool.db_config.database
+          assert_equal "readonly", pool.db_config.name
         ensure
           ActiveRecord::Base.configurations = @prev_configs
           ActiveRecord::Base.establish_connection(:arunit)
@@ -113,8 +118,8 @@ module ActiveRecord
 
           config = {
             "default_env" => {
-              "readonly" => { "adapter" => "sqlite3", "database" => "db/readonly.sqlite3" },
-              "primary"  => { "adapter" => "sqlite3", "database" => "db/primary.sqlite3" }
+              "readonly" => { "adapter" => "sqlite3", "database" => "test/db/readonly.sqlite3" },
+              "primary"  => { "adapter" => "sqlite3", "database" => "test/db/primary.sqlite3" }
             }
           }
           @prev_configs, ActiveRecord::Base.configurations = ActiveRecord::Base.configurations, config
@@ -122,8 +127,6 @@ module ActiveRecord
           ActiveRecord::Base.connects_to(database: { writing: :primary, reading: :readonly })
 
           ActiveRecord::Base.connected_to(role: :reading) do
-            @ro_handler = ActiveRecord::Base.connection_handler
-            assert_equal ActiveRecord::Base.connection_handler, ActiveRecord::Base.connection_handlers[:reading]
             assert_equal :reading, ActiveRecord::Base.current_role
             assert ActiveRecord::Base.connected_to?(role: :reading)
             assert_not ActiveRecord::Base.connected_to?(role: :writing)
@@ -131,8 +134,6 @@ module ActiveRecord
           end
 
           ActiveRecord::Base.connected_to(role: :writing) do
-            assert_equal ActiveRecord::Base.connection_handler, ActiveRecord::Base.connection_handlers[:writing]
-            assert_not_equal @ro_handler, ActiveRecord::Base.connection_handler
             assert_equal :writing, ActiveRecord::Base.current_role
             assert ActiveRecord::Base.connected_to?(role: :writing)
             assert_not ActiveRecord::Base.connected_to?(role: :reading)
@@ -149,19 +150,19 @@ module ActiveRecord
 
           config = {
             "default_env" => {
-              "readonly" => { "adapter" => "sqlite3", "database" => "db/readonly.sqlite3" },
-              "primary"  => { "adapter" => "sqlite3", "database" => "db/primary.sqlite3" }
+              "readonly" => { "adapter" => "sqlite3", "database" => "test/db/readonly.sqlite3" },
+              "primary"  => { "adapter" => "sqlite3", "database" => "test/db/primary.sqlite3" }
             }
           }
           @prev_configs, ActiveRecord::Base.configurations = ActiveRecord::Base.configurations, config
 
           ActiveRecord::Base.connects_to(database: { default: :primary, readonly: :readonly })
 
-          assert_not_nil pool = ActiveRecord::Base.connection_handlers[:default].retrieve_connection_pool("primary")
-          assert_equal "db/primary.sqlite3", pool.spec.config[:database]
+          assert_not_nil pool = ActiveRecord::Base.connection_handler.retrieve_connection_pool("ActiveRecord::Base", role: :default)
+          assert_equal "test/db/primary.sqlite3", pool.db_config.database
 
-          assert_not_nil pool = ActiveRecord::Base.connection_handlers[:readonly].retrieve_connection_pool("primary")
-          assert_equal "db/readonly.sqlite3", pool.spec.config[:database]
+          assert_not_nil pool = ActiveRecord::Base.connection_handler.retrieve_connection_pool("ActiveRecord::Base", role: :readonly)
+          assert_equal "test/db/readonly.sqlite3", pool.db_config.database
         ensure
           ActiveRecord::Base.configurations = @prev_configs
           ActiveRecord::Base.establish_connection(:arunit)
@@ -172,16 +173,15 @@ module ActiveRecord
           previous_env, ENV["RAILS_ENV"] = ENV["RAILS_ENV"], "default_env"
           previous_url, ENV["DATABASE_URL"] = ENV["DATABASE_URL"], "postgres://localhost/foo"
 
-          ActiveRecord::Base.connected_to(database: { writing: "postgres://localhost/bar" }) do
-            assert_equal :writing, ActiveRecord::Base.current_role
-            assert ActiveRecord::Base.connected_to?(role: :writing)
+          ActiveRecord::Base.connects_to(database: { writing: "postgres://localhost/bar" })
+          assert_equal :writing, ActiveRecord::Base.current_role
+          assert ActiveRecord::Base.connected_to?(role: :writing)
 
-            handler = ActiveRecord::Base.connection_handler
-            assert_equal handler, ActiveRecord::Base.connection_handlers[:writing]
+          handler = ActiveRecord::Base.connection_handler
+          assert_equal handler, ActiveRecord::Base.connection_handler
 
-            assert_not_nil pool = handler.retrieve_connection_pool("primary")
-            assert_equal({ adapter: "postgresql", database: "bar", host: "localhost" }, pool.spec.config)
-          end
+          assert_not_nil pool = handler.retrieve_connection_pool("ActiveRecord::Base")
+          assert_equal({ adapter: "postgresql", database: "bar", host: "localhost" }, pool.db_config.configuration_hash)
         ensure
           ActiveRecord::Base.establish_connection(:arunit)
           ENV["RAILS_ENV"] = previous_env
@@ -190,18 +190,17 @@ module ActiveRecord
 
         def test_switching_connections_with_database_config_hash
           previous_env, ENV["RAILS_ENV"] = ENV["RAILS_ENV"], "default_env"
-          config = { adapter: "sqlite3", database: "db/readonly.sqlite3" }
+          config = { adapter: "sqlite3", database: "test/db/readonly.sqlite3" }
 
-          ActiveRecord::Base.connected_to(database: { writing: config }) do
-            assert_equal :writing, ActiveRecord::Base.current_role
-            assert ActiveRecord::Base.connected_to?(role: :writing)
+          ActiveRecord::Base.connects_to(database: { writing: config })
+          assert_equal :writing, ActiveRecord::Base.current_role
+          assert ActiveRecord::Base.connected_to?(role: :writing)
 
-            handler = ActiveRecord::Base.connection_handler
-            assert_equal handler, ActiveRecord::Base.connection_handlers[:writing]
+          handler = ActiveRecord::Base.connection_handler
+          assert_equal handler, ActiveRecord::Base.connection_handler
 
-            assert_not_nil pool = handler.retrieve_connection_pool("primary")
-            assert_equal(config, pool.spec.config)
-          end
+          assert_not_nil pool = handler.retrieve_connection_pool("ActiveRecord::Base")
+          assert_equal(config, pool.db_config.configuration_hash)
         ensure
           ActiveRecord::Base.establish_connection(:arunit)
           ENV["RAILS_ENV"] = previous_env
@@ -209,16 +208,26 @@ module ActiveRecord
 
         def test_switching_connections_with_database_and_role_raises
           error = assert_raises(ArgumentError) do
-            ActiveRecord::Base.connected_to(database: :readonly, role: :writing) { }
+            assert_deprecated do
+              ActiveRecord::Base.connected_to(database: :readonly, role: :writing) { }
+            end
           end
-          assert_equal "connected_to can only accept a `database` or a `role` argument, but not both arguments.", error.message
+          assert_equal "`connected_to` cannot accept a `database` argument with any other arguments.", error.message
+        end
+
+        def test_database_argument_is_deprecated
+          assert_deprecated do
+            ActiveRecord::Base.connected_to(database: { writing: { adapter: "sqlite3", database: "test/db/primary.sqlite3" } }) { }
+          end
+        ensure
+          ActiveRecord::Base.establish_connection(:arunit)
         end
 
         def test_switching_connections_without_database_and_role_raises
           error = assert_raises(ArgumentError) do
             ActiveRecord::Base.connected_to { }
           end
-          assert_equal "must provide a `database` or a `role`.", error.message
+          assert_equal "must provide a `shard` and/or `role`.", error.message
         end
 
         def test_switching_connections_with_database_symbol_uses_default_role
@@ -226,22 +235,21 @@ module ActiveRecord
 
           config = {
             "default_env" => {
-              "animals" => { adapter: "sqlite3", database: "db/animals.sqlite3" },
-              "primary" => { adapter: "sqlite3", database: "db/primary.sqlite3" }
+              "animals" => { adapter: "sqlite3", database: "test/db/animals.sqlite3" },
+              "primary" => { adapter: "sqlite3", database: "test/db/primary.sqlite3" }
             }
           }
           @prev_configs, ActiveRecord::Base.configurations = ActiveRecord::Base.configurations, config
 
-          ActiveRecord::Base.connected_to(database: :animals) do
-            assert_equal :writing, ActiveRecord::Base.current_role
-            assert ActiveRecord::Base.connected_to?(role: :writing)
+          ActiveRecord::Base.connects_to(database: { writing: :animals })
+          assert_equal :writing, ActiveRecord::Base.current_role
+          assert ActiveRecord::Base.connected_to?(role: :writing)
 
-            handler = ActiveRecord::Base.connection_handler
-            assert_equal handler, ActiveRecord::Base.connection_handlers[:writing]
+          handler = ActiveRecord::Base.connection_handler
+          assert_equal handler, ActiveRecord::Base.connection_handler
 
-            assert_not_nil pool = handler.retrieve_connection_pool("primary")
-            assert_equal(config["default_env"]["animals"], pool.spec.config)
-          end
+          assert_not_nil pool = handler.retrieve_connection_pool("ActiveRecord::Base")
+          assert_equal(config["default_env"]["animals"], pool.db_config.configuration_hash)
         ensure
           ActiveRecord::Base.configurations = @prev_configs
           ActiveRecord::Base.establish_connection(:arunit)
@@ -253,22 +261,21 @@ module ActiveRecord
 
           config = {
             "default_env" => {
-              "animals" => { adapter: "sqlite3", database: "db/animals.sqlite3" },
-              "primary" => { adapter: "sqlite3", database: "db/primary.sqlite3" }
+              "animals" => { adapter: "sqlite3", database: "test/db/animals.sqlite3" },
+              "primary" => { adapter: "sqlite3", database: "test/db/primary.sqlite3" }
             }
           }
           @prev_configs, ActiveRecord::Base.configurations = ActiveRecord::Base.configurations, config
 
-          ActiveRecord::Base.connected_to(database: { writing: :primary }) do
-            assert_equal :writing, ActiveRecord::Base.current_role
-            assert ActiveRecord::Base.connected_to?(role: :writing)
+          ActiveRecord::Base.connects_to(database: { writing: :primary })
+          assert_equal :writing, ActiveRecord::Base.current_role
+          assert ActiveRecord::Base.connected_to?(role: :writing)
 
-            handler = ActiveRecord::Base.connection_handler
-            assert_equal handler, ActiveRecord::Base.connection_handlers[:writing]
+          handler = ActiveRecord::Base.connection_handler
+          assert_equal handler, ActiveRecord::Base.connection_handler
 
-            assert_not_nil pool = handler.retrieve_connection_pool("primary")
-            assert_equal(config["default_env"]["primary"], pool.spec.config)
-          end
+          assert_not_nil pool = handler.retrieve_connection_pool("ActiveRecord::Base")
+          assert_equal(config["default_env"]["primary"], pool.db_config.configuration_hash)
         ensure
           ActiveRecord::Base.configurations = @prev_configs
           ActiveRecord::Base.establish_connection(:arunit)
@@ -277,14 +284,13 @@ module ActiveRecord
 
         def test_connects_to_with_single_configuration
           config = {
-            "development" => { "adapter" => "sqlite3", "database" => "db/primary.sqlite3" },
+            "development" => { "adapter" => "sqlite3", "database" => "test/db/primary.sqlite3" },
           }
           @prev_configs, ActiveRecord::Base.configurations = ActiveRecord::Base.configurations, config
 
           ActiveRecord::Base.connects_to database: { writing: :development }
 
-          assert_equal 1, ActiveRecord::Base.connection_handlers.size
-          assert_equal ActiveRecord::Base.connection_handler, ActiveRecord::Base.connection_handlers[:writing]
+          assert_equal ActiveRecord::Base.connection_handler, ActiveRecord::Base.connection_handler
           assert_equal :writing, ActiveRecord::Base.current_role
           assert ActiveRecord::Base.connected_to?(role: :writing)
         ensure
@@ -294,15 +300,15 @@ module ActiveRecord
 
         def test_connects_to_using_top_level_key_in_two_level_config
           config = {
-            "development" => { "adapter" => "sqlite3", "database" => "db/primary.sqlite3" },
-            "development_readonly" => { "adapter" => "sqlite3", "database" => "db/readonly.sqlite3" }
+            "development" => { "adapter" => "sqlite3", "database" => "test/db/primary.sqlite3" },
+            "development_readonly" => { "adapter" => "sqlite3", "database" => "test/db/readonly.sqlite3" }
           }
           @prev_configs, ActiveRecord::Base.configurations = ActiveRecord::Base.configurations, config
 
           ActiveRecord::Base.connects_to database: { writing: :development, reading: :development_readonly }
 
-          assert_not_nil pool = ActiveRecord::Base.connection_handlers[:reading].retrieve_connection_pool("primary")
-          assert_equal "db/readonly.sqlite3", pool.spec.config[:database]
+          assert_not_nil pool = ActiveRecord::Base.connection_handler.retrieve_connection_pool("ActiveRecord::Base", role: :reading)
+          assert_equal "test/db/readonly.sqlite3", pool.db_config.database
         ensure
           ActiveRecord::Base.configurations = @prev_configs
           ActiveRecord::Base.establish_connection(:arunit)
@@ -310,8 +316,8 @@ module ActiveRecord
 
         def test_connects_to_returns_array_of_established_connections
           config = {
-            "development" => { "adapter" => "sqlite3", "database" => "db/primary.sqlite3" },
-            "development_readonly" => { "adapter" => "sqlite3", "database" => "db/readonly.sqlite3" }
+            "development" => { "adapter" => "sqlite3", "database" => "test/db/primary.sqlite3" },
+            "development_readonly" => { "adapter" => "sqlite3", "database" => "test/db/readonly.sqlite3" }
           }
           @prev_configs, ActiveRecord::Base.configurations = ActiveRecord::Base.configurations, config
 
@@ -319,8 +325,8 @@ module ActiveRecord
 
           assert_equal(
             [
-              ActiveRecord::Base.connection_handlers[:writing].retrieve_connection_pool("primary"),
-              ActiveRecord::Base.connection_handlers[:reading].retrieve_connection_pool("primary")
+              ActiveRecord::Base.connection_handler.retrieve_connection_pool("ActiveRecord::Base"),
+              ActiveRecord::Base.connection_handler.retrieve_connection_pool("ActiveRecord::Base", role: :reading)
             ],
             result
           )
@@ -331,87 +337,45 @@ module ActiveRecord
       end
 
       def test_connection_pools
-        assert_equal([@rw_pool], @handlers[:writing].connection_pools)
-        assert_equal([@ro_pool], @handlers[:reading].connection_pools)
+        assert_equal([@rw_pool], @handler.connection_pools(:writing))
+        assert_equal([@ro_pool], @handler.connection_pools(:reading))
       end
 
       def test_retrieve_connection
-        assert @rw_handler.retrieve_connection(@spec_name)
-        assert @ro_handler.retrieve_connection(@spec_name)
+        assert @handler.retrieve_connection(@owner_name)
+        assert @handler.retrieve_connection(@owner_name, role: :reading)
       end
 
       def test_active_connections?
-        assert_not_predicate @rw_handler, :active_connections?
-        assert_not_predicate @ro_handler, :active_connections?
+        assert_not_predicate @handler, :active_connections?
 
-        assert @rw_handler.retrieve_connection(@spec_name)
-        assert @ro_handler.retrieve_connection(@spec_name)
+        assert @handler.retrieve_connection(@owner_name)
+        assert @handler.retrieve_connection(@owner_name, role: :reading)
 
-        assert_predicate @rw_handler, :active_connections?
-        assert_predicate @ro_handler, :active_connections?
+        assert_predicate @handler, :active_connections?
 
-        @rw_handler.clear_active_connections!
-        assert_not_predicate @rw_handler, :active_connections?
-
-        @ro_handler.clear_active_connections!
-        assert_not_predicate @ro_handler, :active_connections?
+        @handler.clear_active_connections!
+        assert_not_predicate @handler, :active_connections?
       end
 
       def test_retrieve_connection_pool
-        assert_not_nil @rw_handler.retrieve_connection_pool(@spec_name)
-        assert_not_nil @ro_handler.retrieve_connection_pool(@spec_name)
+        assert_not_nil @handler.retrieve_connection_pool(@owner_name)
+        assert_not_nil @handler.retrieve_connection_pool(@owner_name, role: :reading)
       end
 
       def test_retrieve_connection_pool_with_invalid_id
-        assert_nil @rw_handler.retrieve_connection_pool("foo")
-        assert_nil @ro_handler.retrieve_connection_pool("foo")
-      end
-
-      def test_connection_handlers_are_per_thread_and_not_per_fiber
-        original_handlers = ActiveRecord::Base.connection_handlers
-
-        ActiveRecord::Base.connection_handlers = { writing: ActiveRecord::Base.default_connection_handler, reading: ActiveRecord::ConnectionAdapters::ConnectionHandler.new }
-
-        reading_handler = ActiveRecord::Base.connection_handlers[:reading]
-
-        reading = ActiveRecord::Base.with_handler(:reading) do
-          Person.connection_handler
-        end
-
-        assert_not_equal reading, ActiveRecord::Base.connection_handler
-        assert_equal reading, reading_handler
-      ensure
-        ActiveRecord::Base.connection_handlers = original_handlers
-      end
-
-      def test_connection_handlers_swapping_connections_in_fiber
-        original_handlers = ActiveRecord::Base.connection_handlers
-
-        ActiveRecord::Base.connection_handlers = { writing: ActiveRecord::Base.default_connection_handler, reading: ActiveRecord::ConnectionAdapters::ConnectionHandler.new }
-
-        reading_handler = ActiveRecord::Base.connection_handlers[:reading]
-
-        enum = Enumerator.new do |r|
-          r << ActiveRecord::Base.connection_handler
-        end
-
-        reading = ActiveRecord::Base.with_handler(:reading) do
-          enum.next
-        end
-
-        assert_equal reading, reading_handler
-      ensure
-        ActiveRecord::Base.connection_handlers = original_handlers
+        assert_nil @handler.retrieve_connection_pool("foo")
+        assert_nil @handler.retrieve_connection_pool("foo", role: :reading)
       end
 
       def test_calling_connected_to_on_a_non_existent_handler_raises
         error = assert_raises ActiveRecord::ConnectionNotEstablished do
-          ActiveRecord::Base.connected_to(role: :reading) do
+          ActiveRecord::Base.connected_to(role: :non_existent) do
             Person.first
           end
         end
 
-        assert_equal "No connection pool with 'primary' found for the 'reading' role.", error.message
+        assert_equal "No connection pool for 'ActiveRecord::Base' found for the 'non_existent' role.", error.message
       end
 
       def test_default_handlers_are_writing_and_reading
