@@ -8,6 +8,7 @@ require "active_record"
 require "cases/test_case"
 require "active_support/dependencies"
 require "active_support/logger"
+require "active_support/core_ext/kernel/singleton_class"
 
 require "support/config"
 require "support/connection"
@@ -37,11 +38,7 @@ end
 
 def in_memory_db?
   current_adapter?(:SQLite3Adapter) &&
-  ActiveRecord::Base.connection_pool.spec.config[:database] == ":memory:"
-end
-
-def subsecond_precision_supported?
-  ActiveRecord::Base.connection.supports_datetime_with_precision?
+  ActiveRecord::Base.connection_pool.db_config.database == ":memory:"
 end
 
 def mysql_enforcing_gtid_consistency?
@@ -61,11 +58,13 @@ end
   supports_savepoints?
   supports_partial_index?
   supports_partitioned_indexes?
+  supports_expression_index?
   supports_insert_returning?
   supports_insert_on_duplicate_skip?
   supports_insert_on_duplicate_update?
   supports_insert_conflict_target?
   supports_optimizer_hints?
+  supports_datetime_with_precision?
 ].each do |method_name|
   define_method method_name do
     ActiveRecord::Base.connection.public_send(method_name)
@@ -150,6 +149,27 @@ def disable_extension!(extension, connection)
   connection.reconnect!
 end
 
+def clean_up_legacy_connection_handlers
+  handler = ActiveRecord::Base.default_connection_handler
+  ActiveRecord::Base.connection_handlers = {}
+
+  handler.connection_pool_names.each do |name|
+    next if ["ActiveRecord::Base", "ARUnit2Model", "Contact", "ContactSti", "FirstAbstractClass", "SecondAbstractClass"].include?(name)
+
+    handler.send(:owner_to_pool_manager).delete(name)
+  end
+end
+
+def clean_up_connection_handler
+  handler = ActiveRecord::Base.connection_handler
+  handler.instance_variable_get(:@owner_to_pool_manager).each do |owner, pool_manager|
+    pool_manager.role_names.each do |role_name|
+      next if role_name == ActiveRecord::Base.default_role
+      pool_manager.remove_role(role_name)
+    end
+  end
+end
+
 def load_schema
   # silence verbose schema loading
   original_stdout = $stdout
@@ -202,5 +222,3 @@ module InTimeZone
       ActiveRecord::Base.time_zone_aware_attributes = old_tz
     end
 end
-
-require_relative "../../../tools/test_common"
