@@ -32,7 +32,7 @@ if ActiveRecord::Base.connection.supports_foreign_keys?
         end
 
         class CreateRocketsMigration < ActiveRecord::Migration::Current
-          def up
+          def change
             create_table :rockets do |t|
               t.string :name
             end
@@ -42,11 +42,6 @@ if ActiveRecord::Base.connection.supports_foreign_keys?
               t.references :rocket, foreign_key: true
             end
           end
-
-          def down
-            drop_table :astronauts, if_exists: true
-            drop_table :rockets, if_exists: true
-          end
         end
 
         def setup
@@ -98,6 +93,10 @@ if ActiveRecord::Base.connection.supports_foreign_keys?
         end
 
         def test_rename_reference_column_of_child_table
+          if current_adapter?(:Mysql2Adapter) && !@connection.send(:supports_rename_index?)
+            skip "Cannot drop index, needed in a foreign key constraint"
+          end
+
           rocket = Rocket.create!(name: "myrocket")
           rocket.astronauts << Astronaut.create!
 
@@ -190,8 +189,8 @@ if ActiveRecord::Base.connection.supports_foreign_keys?
         end
 
         teardown do
-          @connection.drop_table "astronauts", if_exists: true
-          @connection.drop_table "rockets", if_exists: true
+          @connection.drop_table "astronauts", if_exists: true rescue nil
+          @connection.drop_table "rockets", if_exists: true rescue nil
         end
 
         def test_foreign_keys
@@ -309,6 +308,20 @@ if ActiveRecord::Base.connection.supports_foreign_keys?
           assert_equal :nullify, fk.on_update
         end
 
+        def test_add_foreign_key_with_non_existent_from_table_raises
+          e = assert_raises StatementInvalid do
+            @connection.add_foreign_key :missions, :rockets
+          end
+          assert_match(/missions/, e.message)
+        end
+
+        def test_add_foreign_key_with_non_existent_to_table_raises
+          e = assert_raises StatementInvalid do
+            @connection.add_foreign_key :missions, :rockets
+          end
+          assert_match(/missions/, e.message)
+        end
+
         def test_foreign_key_exists
           @connection.add_foreign_key :astronauts, :rockets
 
@@ -533,18 +546,13 @@ if ActiveRecord::Base.connection.supports_foreign_keys?
         end
 
         class CreateSchoolsAndClassesMigration < ActiveRecord::Migration::Current
-          def up
+          def change
             create_table(:schools)
 
             create_table(:classes) do |t|
               t.references :school
             end
-            add_foreign_key :classes, :schools
-          end
-
-          def down
-            drop_table :classes, if_exists: true
-            drop_table :schools, if_exists: true
+            add_foreign_key :classes, :schools, validate: true
           end
         end
 
