@@ -7,7 +7,7 @@ require "models/ship_part"
 require "models/bird"
 require "models/parrot"
 require "models/treasure"
-require "models/man"
+require "models/human"
 require "models/interest"
 require "models/owner"
 require "models/pet"
@@ -129,7 +129,7 @@ class TestNestedAttributesInGeneral < ActiveRecord::TestCase
     pirate = Pirate.new
     ship_built_first = pirate.build_ship
     pirate.ship_attributes = { name: "Ship 1" }
-    assert_equal ship_built_first.object_id, pirate.ship.object_id
+    assert_same ship_built_first, pirate.ship
   end
 
   def test_do_not_allow_assigning_foreign_key_when_reusing_existing_new_record
@@ -140,19 +140,19 @@ class TestNestedAttributesInGeneral < ActiveRecord::TestCase
   end
 
   def test_reject_if_with_a_proc_which_returns_true_always_for_has_many
-    Man.accepts_nested_attributes_for :interests, reject_if: proc { |attributes| true }
-    man = Man.create(name: "John")
-    interest = man.interests.create(topic: "photography")
-    man.update(interests_attributes: { topic: "gardening", id: interest.id })
+    Human.accepts_nested_attributes_for :interests, reject_if: proc { |attributes| true }
+    human = Human.create(name: "John")
+    interest = human.interests.create(topic: "photography")
+    human.update(interests_attributes: { topic: "gardening", id: interest.id })
     assert_equal "photography", interest.reload.topic
   end
 
   def test_destroy_works_independent_of_reject_if
-    Man.accepts_nested_attributes_for :interests, reject_if: proc { |attributes| true }, allow_destroy: true
-    man = Man.create(name: "Jon")
-    interest = man.interests.create(topic: "the ladies")
-    man.update(interests_attributes: { _destroy: "1", id: interest.id })
-    assert_empty man.reload.interests
+    Human.accepts_nested_attributes_for :interests, reject_if: proc { |attributes| true }, allow_destroy: true
+    human = Human.create(name: "Jon")
+    interest = human.interests.create(topic: "the ladies")
+    human.update(interests_attributes: { _destroy: "1", id: interest.id })
+    assert_empty human.reload.interests
   end
 
   def test_reject_if_is_not_short_circuited_if_allow_destroy_is_false
@@ -169,10 +169,10 @@ class TestNestedAttributesInGeneral < ActiveRecord::TestCase
   end
 
   def test_has_many_association_updating_a_single_record
-    Man.accepts_nested_attributes_for(:interests)
-    man = Man.create(name: "John")
-    interest = man.interests.create(topic: "photography")
-    man.update(interests_attributes: { topic: "gardening", id: interest.id })
+    Human.accepts_nested_attributes_for(:interests)
+    human = Human.create(name: "John")
+    interest = human.interests.create(topic: "photography")
+    human.update(interests_attributes: { topic: "gardening", id: interest.id })
     assert_equal "gardening", interest.reload.topic
   end
 
@@ -186,12 +186,12 @@ class TestNestedAttributesInGeneral < ActiveRecord::TestCase
   end
 
   def test_first_and_array_index_zero_methods_return_the_same_value_when_nested_attributes_are_set_to_update_existing_record
-    Man.accepts_nested_attributes_for(:interests)
-    man = Man.create(name: "John")
-    interest = man.interests.create topic: "gardening"
-    man = Man.find man.id
-    man.interests_attributes = [{ id: interest.id, topic: "gardening" }]
-    assert_equal man.interests.first.topic, man.interests[0].topic
+    Human.accepts_nested_attributes_for(:interests)
+    human = Human.create(name: "John")
+    interest = human.interests.create topic: "gardening"
+    human = Human.find human.id
+    human.interests_attributes = [{ id: interest.id, topic: "gardening" }]
+    assert_equal human.interests.first.topic, human.interests[0].topic
   end
 
   def test_allows_class_to_override_setter_and_call_super
@@ -219,10 +219,10 @@ class TestNestedAttributesInGeneral < ActiveRecord::TestCase
   end
 
   def test_should_not_create_duplicates_with_create_with
-    Man.accepts_nested_attributes_for(:interests)
+    Human.accepts_nested_attributes_for(:interests)
 
     assert_difference("Interest.count", 1) do
-      Man.create_with(
+      Human.create_with(
         interests_attributes: [{ topic: "Pirate king" }]
       ).find_or_create_by!(
         name: "Monkey D. Luffy"
@@ -614,7 +614,7 @@ module NestedAttributesOnACollectionAssociationTests
       :catchphrase => "Arr",
       association_getter => { "foo" => { name: "Grace OMalley" } })
 
-    assert_equal 1, pirate.reload.send(@association_name).count
+    assert_equal 1, pirate.reload.public_send(@association_name).count
   end
 
   def test_should_take_a_hash_with_string_keys_and_assign_the_attributes_to_the_associated_models
@@ -624,58 +624,58 @@ module NestedAttributesOnACollectionAssociationTests
   end
 
   def test_should_take_an_array_and_assign_the_attributes_to_the_associated_models
-    @pirate.send(association_setter, @alternate_params[association_getter].values)
+    @pirate.public_send(association_setter, @alternate_params[association_getter].values)
     @pirate.save
     assert_equal ["Grace OMalley", "Privateers Greed"], [@child_1.reload.name, @child_2.reload.name]
   end
 
   def test_should_also_work_with_a_HashWithIndifferentAccess
-    @pirate.send(association_setter, ActiveSupport::HashWithIndifferentAccess.new("foo" => ActiveSupport::HashWithIndifferentAccess.new(id: @child_1.id, name: "Grace OMalley")))
+    @pirate.public_send(association_setter, ActiveSupport::HashWithIndifferentAccess.new("foo" => ActiveSupport::HashWithIndifferentAccess.new(id: @child_1.id, name: "Grace OMalley")))
     @pirate.save
     assert_equal "Grace OMalley", @child_1.reload.name
   end
 
   def test_should_take_a_hash_and_assign_the_attributes_to_the_associated_models
     @pirate.attributes = @alternate_params
-    assert_equal "Grace OMalley", @pirate.send(@association_name).first.name
-    assert_equal "Privateers Greed", @pirate.send(@association_name).last.name
+    assert_equal "Grace OMalley", @pirate.public_send(@association_name).first.name
+    assert_equal "Privateers Greed", @pirate.public_send(@association_name).last.name
   end
 
   def test_should_not_load_association_when_updating_existing_records
     @pirate.reload
-    @pirate.send(association_setter, [{ id: @child_1.id, name: "Grace OMalley" }])
-    assert_not_predicate @pirate.send(@association_name), :loaded?
+    @pirate.public_send(association_setter, [{ id: @child_1.id, name: "Grace OMalley" }])
+    assert_not_predicate @pirate.public_send(@association_name), :loaded?
 
     @pirate.save
-    assert_not_predicate @pirate.send(@association_name), :loaded?
+    assert_not_predicate @pirate.public_send(@association_name), :loaded?
     assert_equal "Grace OMalley", @child_1.reload.name
   end
 
   def test_should_not_overwrite_unsaved_updates_when_loading_association
     @pirate.reload
-    @pirate.send(association_setter, [{ id: @child_1.id, name: "Grace OMalley" }])
-    assert_equal "Grace OMalley", @pirate.send(@association_name).load_target.find { |r| r.id == @child_1.id }.name
+    @pirate.public_send(association_setter, [{ id: @child_1.id, name: "Grace OMalley" }])
+    assert_equal "Grace OMalley", @pirate.public_send(@association_name).load_target.find { |r| r.id == @child_1.id }.name
   end
 
   def test_should_preserve_order_when_not_overwriting_unsaved_updates
     @pirate.reload
-    @pirate.send(association_setter, [{ id: @child_1.id, name: "Grace OMalley" }])
-    assert_equal @child_1.id, @pirate.send(@association_name).load_target.first.id
+    @pirate.public_send(association_setter, [{ id: @child_1.id, name: "Grace OMalley" }])
+    assert_equal @child_1.id, @pirate.public_send(@association_name).load_target.first.id
   end
 
   def test_should_refresh_saved_records_when_not_overwriting_unsaved_updates
     @pirate.reload
     record = @pirate.class.reflect_on_association(@association_name).klass.new(name: "Grace OMalley")
-    @pirate.send(@association_name) << record
+    @pirate.public_send(@association_name) << record
     record.save!
-    @pirate.send(@association_name).last.update!(name: "Polly")
-    assert_equal "Polly", @pirate.send(@association_name).load_target.last.name
+    @pirate.public_send(@association_name).last.update!(name: "Polly")
+    assert_equal "Polly", @pirate.public_send(@association_name).load_target.last.name
   end
 
   def test_should_not_remove_scheduled_destroys_when_loading_association
     @pirate.reload
-    @pirate.send(association_setter, [{ id: @child_1.id, _destroy: "1" }])
-    assert_predicate @pirate.send(@association_name).load_target.find { |r| r.id == @child_1.id }, :marked_for_destruction?
+    @pirate.public_send(association_setter, [{ id: @child_1.id, _destroy: "1" }])
+    assert_predicate @pirate.public_send(@association_name).load_target.find { |r| r.id == @child_1.id }, :marked_for_destruction?
   end
 
   def test_should_take_a_hash_with_composite_id_keys_and_assign_the_attributes_to_the_associated_models
@@ -702,7 +702,7 @@ module NestedAttributesOnACollectionAssociationTests
 
   def test_should_raise_RecordNotFound_if_an_id_belonging_to_a_different_record_is_given
     other_pirate = Pirate.create! catchphrase: "Ahoy!"
-    other_child = other_pirate.send(@association_name).create! name: "Buccaneers Servant"
+    other_child = other_pirate.public_send(@association_name).create! name: "Buccaneers Servant"
 
     exception = assert_raise ActiveRecord::RecordNotFound do
       @pirate.attributes = { association_getter => [{ id: other_child.id }] }
@@ -711,26 +711,26 @@ module NestedAttributesOnACollectionAssociationTests
   end
 
   def test_should_automatically_build_new_associated_models_for_each_entry_in_a_hash_where_the_id_is_missing
-    @pirate.send(@association_name).destroy_all
+    @pirate.public_send(@association_name).destroy_all
     @pirate.reload.attributes = {
       association_getter => { "foo" => { name: "Grace OMalley" }, "bar" => { name: "Privateers Greed" } }
     }
 
-    assert_not_predicate @pirate.send(@association_name).first, :persisted?
-    assert_equal "Grace OMalley", @pirate.send(@association_name).first.name
+    assert_not_predicate @pirate.public_send(@association_name).first, :persisted?
+    assert_equal "Grace OMalley", @pirate.public_send(@association_name).first.name
 
-    assert_not_predicate @pirate.send(@association_name).last, :persisted?
-    assert_equal "Privateers Greed", @pirate.send(@association_name).last.name
+    assert_not_predicate @pirate.public_send(@association_name).last, :persisted?
+    assert_equal "Privateers Greed", @pirate.public_send(@association_name).last.name
   end
 
   def test_should_not_assign_destroy_key_to_a_record
     assert_nothing_raised do
-      @pirate.send(association_setter, "foo" => { "_destroy" => "0" })
+      @pirate.public_send(association_setter, "foo" => { "_destroy" => "0" })
     end
   end
 
   def test_should_ignore_new_associated_records_with_truthy_destroy_attribute
-    @pirate.send(@association_name).destroy_all
+    @pirate.public_send(@association_name).destroy_all
     @pirate.reload.attributes = {
       association_getter => {
         "foo" => { name: "Grace OMalley" },
@@ -738,13 +738,13 @@ module NestedAttributesOnACollectionAssociationTests
       }
     }
 
-    assert_equal 1, @pirate.send(@association_name).length
-    assert_equal "Grace OMalley", @pirate.send(@association_name).first.name
+    assert_equal 1, @pirate.public_send(@association_name).length
+    assert_equal "Grace OMalley", @pirate.public_send(@association_name).first.name
   end
 
   def test_should_ignore_new_associated_records_if_a_reject_if_proc_returns_false
     @alternate_params[association_getter]["baz"] = {}
-    assert_no_difference("@pirate.send(@association_name).count") do
+    assert_no_difference("@pirate.public_send(@association_name).count") do
       @pirate.attributes = @alternate_params
     end
   end
@@ -753,17 +753,17 @@ module NestedAttributesOnACollectionAssociationTests
     attributes = {}
     attributes["123726353"] = { name: "Grace OMalley" }
     attributes["2"] = { name: "Privateers Greed" } # 2 is lower then 123726353
-    @pirate.send(association_setter, attributes)
+    @pirate.public_send(association_setter, attributes)
 
-    assert_equal ["Posideons Killer", "Killer bandita Dionne", "Privateers Greed", "Grace OMalley"].to_set, @pirate.send(@association_name).map(&:name).to_set
+    assert_equal ["Posideons Killer", "Killer bandita Dionne", "Privateers Greed", "Grace OMalley"].to_set, @pirate.public_send(@association_name).map(&:name).to_set
   end
 
   def test_should_raise_an_argument_error_if_something_else_than_a_hash_is_passed
-    assert_nothing_raised { @pirate.send(association_setter, {}) }
-    assert_nothing_raised { @pirate.send(association_setter, Hash.new) }
+    assert_nothing_raised { @pirate.public_send(association_setter, {}) }
+    assert_nothing_raised { @pirate.public_send(association_setter, Hash.new) }
 
     exception = assert_raise ArgumentError do
-      @pirate.send(association_setter, "foo")
+      @pirate.public_send(association_setter, "foo")
     end
     assert_equal %{Hash or Array expected for attribute `#{@association_name}`, got String ("foo")}, exception.message
   end
@@ -777,20 +777,20 @@ module NestedAttributesOnACollectionAssociationTests
 
   def test_should_update_existing_records_and_add_new_ones_that_have_no_id
     @alternate_params[association_getter]["baz"] = { name: "Buccaneers Servant" }
-    assert_difference("@pirate.send(@association_name).count", +1) do
+    assert_difference("@pirate.public_send(@association_name).count", +1) do
       @pirate.update @alternate_params
     end
-    assert_equal ["Grace OMalley", "Privateers Greed", "Buccaneers Servant"].to_set, @pirate.reload.send(@association_name).map(&:name).to_set
+    assert_equal ["Grace OMalley", "Privateers Greed", "Buccaneers Servant"].to_set, @pirate.reload.public_send(@association_name).map(&:name).to_set
   end
 
   def test_should_be_possible_to_destroy_a_record
     ["1", 1, "true", true].each do |true_variable|
-      record = @pirate.reload.send(@association_name).create!(name: "Grace OMalley")
-      @pirate.send(association_setter,
+      record = @pirate.reload.public_send(@association_name).create!(name: "Grace OMalley")
+      @pirate.public_send(association_setter,
         @alternate_params[association_getter].merge("baz" => { :id => record.id, "_destroy" => true_variable })
       )
 
-      assert_difference("@pirate.send(@association_name).count", -1) do
+      assert_difference("@pirate.public_send(@association_name).count", -1) do
         @pirate.save
       end
     end
@@ -799,17 +799,17 @@ module NestedAttributesOnACollectionAssociationTests
   def test_should_not_destroy_the_associated_model_with_a_non_truthy_argument
     [nil, "", "0", 0, "false", false].each do |false_variable|
       @alternate_params[association_getter]["foo"]["_destroy"] = false_variable
-      assert_no_difference("@pirate.send(@association_name).count") do
+      assert_no_difference("@pirate.public_send(@association_name).count") do
         @pirate.update(@alternate_params)
       end
     end
   end
 
   def test_should_not_destroy_the_associated_model_until_the_parent_is_saved
-    assert_no_difference("@pirate.send(@association_name).count") do
-      @pirate.send(association_setter, @alternate_params[association_getter].merge("baz" => { :id => @child_1.id, "_destroy" => true }))
+    assert_no_difference("@pirate.public_send(@association_name).count") do
+      @pirate.public_send(association_setter, @alternate_params[association_getter].merge("baz" => { :id => @child_1.id, "_destroy" => true }))
     end
-    assert_difference("@pirate.send(@association_name).count", -1) { @pirate.save }
+    assert_difference("@pirate.public_send(@association_name).count", -1) { @pirate.save }
   end
 
   def test_should_automatically_enable_autosave_on_the_association
@@ -817,17 +817,17 @@ module NestedAttributesOnACollectionAssociationTests
   end
 
   def test_validate_presence_of_parent_works_with_inverse_of
-    Man.accepts_nested_attributes_for(:interests)
-    assert_equal :man, Man.reflect_on_association(:interests).options[:inverse_of]
-    assert_equal :interests, Interest.reflect_on_association(:man).options[:inverse_of]
+    Human.accepts_nested_attributes_for(:interests)
+    assert_equal :human, Human.reflect_on_association(:interests).options[:inverse_of]
+    assert_equal :interests, Interest.reflect_on_association(:human).options[:inverse_of]
 
     repair_validations(Interest) do
-      Interest.validates_presence_of(:man)
-      assert_difference "Man.count" do
+      Interest.validates_presence_of(:human)
+      assert_difference "Human.count" do
         assert_difference "Interest.count", 2 do
-          man = Man.create!(name: "John",
+          human = Human.create!(name: "John",
                             interests_attributes: [{ topic: "Cars" }, { topic: "Sports" }])
-          assert_equal 2, man.interests.count
+          assert_equal 2, human.interests.count
         end
       end
     end
@@ -839,14 +839,14 @@ module NestedAttributesOnACollectionAssociationTests
   end
 
   def test_numeric_column_changes_from_zero_to_no_empty_string
-    Man.accepts_nested_attributes_for(:interests)
+    Human.accepts_nested_attributes_for(:interests)
 
     repair_validations(Interest) do
       Interest.validates_numericality_of(:zine_id)
-      man = Man.create(name: "John")
-      interest = man.interests.create(topic: "bar", zine_id: 0)
+      human = Human.create(name: "John")
+      interest = human.interests.create(topic: "bar", zine_id: 0)
       assert interest.save
-      assert_not man.update(interests_attributes: { id: interest.id, zine_id: "foo" })
+      assert_not human.update(interests_attributes: { id: interest.id, zine_id: "foo" })
     end
   end
 
