@@ -86,6 +86,7 @@ class PersistenceTest < ActiveRecord::TestCase
 
   def test_increment_attribute
     assert_equal 50, accounts(:signals37).credit_limit
+
     accounts(:signals37).increment! :credit_limit
     assert_equal 51, accounts(:signals37, :reload).credit_limit
 
@@ -93,6 +94,16 @@ class PersistenceTest < ActiveRecord::TestCase
     assert_equal 53, accounts(:signals37, :reload).credit_limit
   end
 
+  def test_increment_aliased_attribute
+    assert_equal 50, accounts(:signals37).available_credit
+
+    accounts(:signals37).increment!(:available_credit)
+    assert_equal 51, accounts(:signals37, :reload).available_credit
+
+    accounts(:signals37).increment(:available_credit).increment!(:available_credit)
+    assert_equal 53, accounts(:signals37, :reload).available_credit
+  end
+
   def test_increment_nil_attribute
     assert_nil topics(:first).parent_id
     topics(:first).increment! :parent_id
@@ -183,7 +194,7 @@ class PersistenceTest < ActiveRecord::TestCase
     assert_not_predicate company, :valid?
     original_errors = company.errors
     client = company.becomes(Client)
-    assert_equal original_errors.keys, client.errors.keys
+    assert_equal assert_deprecated { original_errors.keys }, assert_deprecated { client.errors.keys }
   end
 
   def test_becomes_errors_base
@@ -197,7 +208,7 @@ class PersistenceTest < ActiveRecord::TestCase
     admin.errors.add :token, :invalid
     child = admin.becomes(child_class)
 
-    assert_equal [:token], child.errors.keys
+    assert_equal [:token], assert_deprecated { child.errors.keys }
     assert_nothing_raised do
       child.errors.add :foo, :invalid
     end
@@ -865,13 +876,6 @@ class PersistenceTest < ActiveRecord::TestCase
     assert_equal topic.title, Topic.find(1234).title
   end
 
-  def test_update_attributes
-    topic = Topic.find(1)
-    assert_deprecated do
-      topic.update_attributes("title" => "The First Topic Updated")
-    end
-  end
-
   def test_update_parameters
     topic = Topic.find(1)
     assert_nothing_raised do
@@ -904,13 +908,6 @@ class PersistenceTest < ActiveRecord::TestCase
     Reply.clear_validators!
   end
 
-  def test_update_attributes!
-    reply = Reply.find(2)
-    assert_deprecated do
-      reply.update_attributes!("title" => "The Second Topic of the day updated")
-    end
-  end
-
   def test_destroyed_returns_boolean
     developer = Developer.first
     assert_equal false, developer.destroyed?
