@@ -87,26 +87,39 @@ class ActiveRecord::Relation
       end
     end
 
-    test "invert replaces each part of the predicate with its inverse" do
-      random_object = Object.new
+    test "invert wraps the ast inside a NAND node" do
       original = WhereClause.new([
         table["id"].in([1, 2, 3]),
+        table["id"].not_in([1, 2, 3]),
         table["id"].eq(1),
+        table["id"].not_eq(2),
+        table["id"].gt(1),
+        table["id"].gteq(2),
+        table["id"].lt(1),
+        table["id"].lteq(2),
         table["id"].is_not_distinct_from(1),
         table["id"].is_distinct_from(2),
-        "sql literal",
-        random_object
+        "sql literal"
       ])
       expected = WhereClause.new([
-        table["id"].not_in([1, 2, 3]),
-        table["id"].not_eq(1),
-        table["id"].is_distinct_from(1),
-        table["id"].is_not_distinct_from(2),
-        Arel::Nodes::Not.new(Arel::Nodes::SqlLiteral.new("sql literal")),
-        Arel::Nodes::Not.new(random_object)
+        Arel::Nodes::Not.new(
+          Arel::Nodes::And.new([
+            table["id"].in([1, 2, 3]),
+            table["id"].not_in([1, 2, 3]),
+            table["id"].eq(1),
+            table["id"].not_eq(2),
+            table["id"].gt(1),
+            table["id"].gteq(2),
+            table["id"].lt(1),
+            table["id"].lteq(2),
+            table["id"].is_not_distinct_from(1),
+            table["id"].is_distinct_from(2),
+            Arel::Nodes::Grouping.new("sql literal")
+          ])
+        )
       ])
 
-      assert_equal expected, original.invert(:nor)
+      assert_equal expected, original.invert
     end
 
     test "except removes binary predicates referencing a given column" do
