@@ -26,7 +26,7 @@ module ActiveRecord
     def test_initialize_single_values
       relation = Relation.new(FakeKlass)
       (Relation::SINGLE_VALUE_METHODS - [:create_with]).each do |method|
-        assert_nil relation.send("#{method}_value"), method.to_s
+        assert_nil relation.public_send("#{method}_value"), method.to_s
       end
       value = relation.create_with_value
       assert_equal({}, value)
@@ -36,7 +36,7 @@ module ActiveRecord
     def test_multi_value_initialize
       relation = Relation.new(FakeKlass)
       Relation::MULTI_VALUE_METHODS.each do |method|
-        values = relation.send("#{method}_values")
+        values = relation.public_send("#{method}_values")
         assert_equal [], values, method.to_s
         assert_predicate values, :frozen?, method.to_s
       end
@@ -50,6 +50,19 @@ module ActiveRecord
     def test_empty_where_values_hash
       relation = Relation.new(FakeKlass)
       assert_equal({}, relation.where_values_hash)
+
+      relation.where!(relation.table[:id].not_eq(10))
+      assert_equal({}, relation.where_values_hash)
+
+      relation.where!(relation.table[:id].is_distinct_from(10))
+      assert_equal({}, relation.where_values_hash)
+    end
+
+    def test_where_values_hash_with_in_clause
+      relation = Relation.new(Post)
+      relation.where!(title: ["foo", "bar", "hello"])
+
+      assert_equal({ "title" => ["foo", "bar", "hello"] }, relation.where_values_hash)
     end
 
     def test_has_values
@@ -127,13 +140,13 @@ module ActiveRecord
       relation = Relation.new(FakeKlass)
       assert_equal [], relation.references_values
       relation = relation.references(:foo).references(:omg, :lol)
-      assert_equal ["foo", "omg", "lol"], relation.references_values
+      assert_equal [:foo, :omg, :lol], relation.references_values
     end
 
     def test_references_values_dont_duplicate
       relation = Relation.new(FakeKlass)
       relation = relation.references(:foo).references(:foo)
-      assert_equal ["foo"], relation.references_values
+      assert_equal [:foo], relation.references_values
     end
 
     test "merging a hash into a relation" do
@@ -411,6 +424,38 @@ module ActiveRecord
       end
     end
 
+    def test_marshal_load_legacy_relation
+      path = File.expand_path(
+        "support/marshal_compatibility_fixtures/legacy_relation.dump",
+        TEST_ROOT
+      )
+      assert_equal 11, Marshal.load(File.read(path)).size
+    end
+
+    test "no queries on empty IN" do
+      assert_queries(0) do
+        Post.where(id: []).load
+      end
+    end
+
+    test "can unscope empty IN" do
+      assert_queries(1) do
+        Post.where(id: []).unscope(where: :id).load
+      end
+    end
+
+    test "no queries on empty relation exists?" do
+      assert_queries(0) do
+        Post.where(id: []).exists?(123)
+      end
+    end
+
+    test "no queries on empty condition exists?" do
+      assert_queries(0) do
+        Post.all.exists?(id: [])
+      end
+    end
+
     private
       def skip_if_sqlite3_version_includes_quoting_bug
         if sqlite3_version_includes_quoting_bug?
