@@ -419,7 +419,6 @@ class TransactionCallbacksTest < ActiveRecord::TestCase
     klass = Class.new(TopicWithCallbacks) do
       self.inheritance_column = nil
       validates :title, presence: true
-      def self.name; "TopicWithCallbacks"; end
     end
 
     first = klass.new(title: "foo")
@@ -539,6 +538,18 @@ class TransactionCallbacksTest < ActiveRecord::TestCase
     assert_equal [:after_rollback], records.second.history
   end
 
+  def test_after_commit_does_not_mutate_the_if_options_array
+    opts = []
+
+    Class.new(ActiveRecord::Base) do
+      self.table_name = "topics"
+
+      after_commit(if: opts, on: :create) { }
+    end
+
+    assert_empty opts
+  end
+
   private
     def add_transaction_execution_blocks(record)
       record.after_commit_block(:create) { |r| r.history << :commit_on_create }
@@ -720,75 +731,6 @@ class CallbacksOnDestroyUpdateActionRaceTest < ActiveRecord::TestCase
   end
 end
 
-class TransactionEnrollmentCallbacksTest < ActiveRecord::TestCase
-  class TopicWithoutTransactionalEnrollmentCallbacks < ActiveRecord::Base
-    self.table_name = :topics
-
-    before_commit_without_transaction_enrollment { |r| r.history << :before_commit }
-    after_commit_without_transaction_enrollment { |r| r.history << :after_commit }
-    after_rollback_without_transaction_enrollment { |r| r.history << :rollback }
-
-    def history
-      @history ||= []
-    end
-  end
-
-  def setup
-    @topic = TopicWithoutTransactionalEnrollmentCallbacks.create!
-  end
-
-  def test_commit_does_not_run_transactions_callbacks_without_enrollment
-    @topic.transaction do
-      @topic.content = "foo"
-      @topic.save!
-    end
-    assert_empty @topic.history
-  end
-
-  def test_commit_run_transactions_callbacks_with_explicit_enrollment
-    @topic.transaction do
-      2.times do
-        @topic.content = "foo"
-        @topic.save!
-      end
-      @topic.send(:add_to_transaction)
-    end
-    assert_equal [:before_commit, :after_commit], @topic.history
-  end
-
-  def test_commit_run_transactions_callbacks_with_nested_transactions
-    @topic.transaction do
-      @topic.transaction(requires_new: true) do
-        @topic.content = "foo"
-        @topic.save!
-        @topic.send(:add_to_transaction)
-      end
-    end
-    assert_equal [:before_commit, :after_commit], @topic.history
-  end
-
-  def test_rollback_does_not_run_transactions_callbacks_without_enrollment
-    @topic.transaction do
-      @topic.content = "foo"
-      @topic.save!
-      raise ActiveRecord::Rollback
-    end
-    assert_empty @topic.history
-  end
-
-  def test_rollback_run_transactions_callbacks_with_explicit_enrollment
-    @topic.transaction do
-      2.times do
-        @topic.content = "foo"
-        @topic.save!
-      end
-      @topic.send(:add_to_transaction)
-      raise ActiveRecord::Rollback
-    end
-    assert_equal [:rollback], @topic.history
-  end
-end
-
 class CallbacksOnActionAndConditionTest < ActiveRecord::TestCase
   self.use_transactional_tests = false
 
