@@ -19,26 +19,26 @@ class YamlSerializationTest < ActiveRecord::TestCase
   def test_roundtrip
     topic = Topic.first
     assert topic
-    t = YAML.load YAML.dump topic
+    t = yaml_load YAML.dump topic
     assert_equal topic, t
   end
 
   def test_roundtrip_serialized_column
     topic = Topic.new(content: { omg: :lol })
-    assert_equal({ omg: :lol }, YAML.load(YAML.dump(topic)).content)
+    assert_equal({ omg: :lol }, yaml_load(YAML.dump(topic)).content)
   end
 
   def test_psych_roundtrip
     topic = Topic.first
     assert topic
-    t = Psych.load Psych.dump topic
+    t = yaml_load Psych.dump topic
     assert_equal topic, t
   end
 
   def test_psych_roundtrip_new_object
     topic = Topic.new
     assert topic
-    t = Psych.load Psych.dump topic
+    t = yaml_load Psych.dump topic
     assert_equal topic.attributes, t.attributes
   end
 
@@ -49,30 +49,30 @@ class YamlSerializationTest < ActiveRecord::TestCase
   def test_raw_types_are_not_changed_on_round_trip
     topic = Topic.new(parent_id: "123")
     assert_equal "123", topic.parent_id_before_type_cast
-    assert_equal "123", YAML.load(YAML.dump(topic)).parent_id_before_type_cast
+    assert_equal "123", yaml_load(YAML.dump(topic)).parent_id_before_type_cast
   end
 
   def test_cast_types_are_not_changed_on_round_trip
     topic = Topic.new(parent_id: "123")
     assert_equal 123, topic.parent_id
-    assert_equal 123, YAML.load(YAML.dump(topic)).parent_id
+    assert_equal 123, yaml_load(YAML.dump(topic)).parent_id
   end
 
   def test_new_records_remain_new_after_round_trip
     topic = Topic.new
 
     assert topic.new_record?, "Sanity check that new records are new"
-    assert YAML.load(YAML.dump(topic)).new_record?, "Record should be new after deserialization"
+    assert yaml_load(YAML.dump(topic)).new_record?, "Record should be new after deserialization"
 
     topic.save!
 
     assert_not topic.new_record?, "Saved records are not new"
-    assert_not YAML.load(YAML.dump(topic)).new_record?, "Saved record should not be new after deserialization"
+    assert_not yaml_load(YAML.dump(topic)).new_record?, "Saved record should not be new after deserialization"
 
     topic = Topic.select("title").last
 
     assert_not topic.new_record?, "Loaded records without ID are not new"
-    assert_not YAML.load(YAML.dump(topic)).new_record?, "Record should not be new after deserialization"
+    assert_not yaml_load(YAML.dump(topic)).new_record?, "Record should not be new after deserialization"
   end
 
   def test_types_of_virtual_columns_are_not_changed_on_round_trip
@@ -80,7 +80,7 @@ class YamlSerializationTest < ActiveRecord::TestCase
       .joins(:posts)
       .group("authors.id")
       .first
-    dumped = YAML.load(YAML.dump(author))
+    dumped = yaml_load(YAML.dump(author))
 
     assert_equal 5, author.posts_count
     assert_equal 5, dumped.posts_count
@@ -93,8 +93,28 @@ class YamlSerializationTest < ActiveRecord::TestCase
     assert coder["active_record_yaml_version"]
   end
 
+  def test_deserializing_rails_v2_yaml
+    topic = yaml_load(yaml_fixture("rails_v2"))
+
+    assert_not_predicate topic, :new_record?
+    assert_equal 1, topic.id
+    assert_equal "The First Topic", topic.title
+    assert_equal "Have a nice day", topic.content
+  end
+
+  def test_deserializing_rails_v1_mysql_yaml
+    topic = yaml_load(yaml_fixture("rails_v1_mysql"))
+
+    assert_not_predicate topic, :new_record?
+    assert_equal 1, topic.id
+    assert_equal "The First Topic", topic.title
+    assert_equal "Have a nice day", topic.content
+  end
+
   def test_deserializing_rails_41_yaml
-    topic = YAML.load(yaml_fixture("rails_4_1"))
+    topic = assert_deprecated do
+      yaml_load(yaml_fixture("rails_4_1"))
+    end
 
     assert_predicate topic, :new_record?
     assert_nil topic.id
@@ -103,7 +123,9 @@ class YamlSerializationTest < ActiveRecord::TestCase
   end
 
   def test_deserializing_rails_4_2_0_yaml
-    topic = YAML.load(yaml_fixture("rails_4_2_0"))
+    topic = assert_deprecated do
+      yaml_load(yaml_fixture("rails_4_2_0"))
+    end
 
     assert_not_predicate topic, :new_record?
     assert_equal 1, topic.id
@@ -114,7 +136,7 @@ class YamlSerializationTest < ActiveRecord::TestCase
   def test_yaml_encoding_keeps_mutations
     author = Author.first
     author.name = "Sean"
-    dumped = YAML.load(YAML.dump(author))
+    dumped = yaml_load(YAML.dump(author))
 
     assert_equal "Sean", dumped.name
     assert_equal author.name_was, dumped.name_was
@@ -124,16 +146,20 @@ class YamlSerializationTest < ActiveRecord::TestCase
   def test_yaml_encoding_keeps_false_values
     topic = Topic.first
     topic.approved = false
-    dumped = YAML.load(YAML.dump(topic))
+    dumped = yaml_load(YAML.dump(topic))
 
     assert_equal false, dumped.approved
   end
 
   private
+    def yaml_load(payload)
+      YAML.respond_to?(:unsafe_load) ? YAML.unsafe_load(payload) : YAML.load(payload)
+    end
+
     def yaml_fixture(file_name)
       path = File.expand_path(
-        "../support/yaml_compatibility_fixtures/#{file_name}.yml",
-        __dir__
+        "support/yaml_compatibility_fixtures/#{file_name}.yml",
+        TEST_ROOT
       )
       File.read(path)
     end
