@@ -3,6 +3,7 @@
 class Post < ActiveRecord::Base
   class CategoryPost < ActiveRecord::Base
     self.table_name = "categories_posts"
+    belongs_to :group, foreign_key: :category_id, class_name: "Category"
     belongs_to :category
     belongs_to :post
   end
@@ -23,9 +24,12 @@ class Post < ActiveRecord::Base
     end
   end
 
+  alias_attribute :text, :body
+  alias_attribute :comments_count, :legacy_comments_count
+
   scope :containing_the_letter_a, -> { where("body LIKE '%a%'") }
   scope :titled_with_an_apostrophe, -> { where("title LIKE '%''%'") }
-  scope :ranked_by_comments, -> { order(arel_attribute(:comments_count).desc) }
+  scope :ranked_by_comments, -> { order(table[:comments_count].desc) }
 
   scope :limit_by, lambda { |l| limit(l) }
   scope :locked, -> { lock }
@@ -288,6 +292,7 @@ class PostWithAfterCreateCallback < ActiveRecord::Base
   self.inheritance_column = :disabled
   self.table_name = "posts"
   has_many :comments, foreign_key: :post_id
+  has_and_belongs_to_many :categories, foreign_key: :post_id
 
   after_create do |post|
     update_attribute(:author_id, comments.first.id)
@@ -336,10 +341,6 @@ class FakeKlass
       sql
     end
 
-    def arel_attribute(name, table)
-      table[name]
-    end
-
     def disallow_raw_sql!(*args)
       # noop
     end
@@ -356,6 +357,10 @@ class FakeKlass
       Post.predicate_builder
     end
 
+    def finder_needs_type_condition?
+      false
+    end
+
     def base_class?
       true
     end
