@@ -27,7 +27,8 @@ class ShipWithoutNestedAttributes < ActiveRecord::Base
   has_many :prisoners, inverse_of: :ship, foreign_key: :ship_id
   has_many :parts, class_name: "ShipPart", foreign_key: :ship_id
 
-  validates :name, presence: true
+  validates :name, presence: true, if: -> { true }
+  validates :name, presence: true, if: -> { true }
 end
 
 class Prisoner < ActiveRecord::Base
