@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 ActiveRecord::Schema.define do
-  if subsecond_precision_supported?
+  if supports_datetime_with_precision?
     create_table :datetime_defaults, force: true do |t|
       t.datetime :modified_datetime, default: -> { "CURRENT_TIMESTAMP" }
       t.datetime :precise_datetime, precision: 6, default: -> { "CURRENT_TIMESTAMP(6)" }
