@@ -1,6 +1,13 @@
 # frozen_string_literal: true
 
-require "image_processing"
+begin
+  require "image_processing"
+rescue LoadError
+  raise LoadError, <<~ERROR.squish
+    Generating image variants require the image_processing gem.
+    Please add `gem 'image_processing', '~> 1.2'` to your Gemfile.
+  ERROR
+end
 
 module ActiveStorage
   module Transformers
@@ -22,14 +29,13 @@ module ActiveStorage
         def operations
           transformations.each_with_object([]) do |(name, argument), list|
             if name.to_s == "combine_options"
-              ActiveSupport::Deprecation.warn <<~WARNING.squish
+              raise ArgumentError, <<~ERROR.squish
                 Active Storage's ImageProcessing transformer doesn't support :combine_options,
-                as it always generates a single ImageMagick command. Passing :combine_options will
-                not be supported in Rails 6.1.
-              WARNING
+                as it always generates a single ImageMagick command.
+              ERROR
+            end
 
-              list.concat argument.keep_if { |key, value| value.present? }.to_a
-            elsif argument.present?
+            if argument.present?
               list << [ name, argument ]
             end
           end
