@@ -30,18 +30,45 @@ module CacheInstrumentationBehavior
 
     assert_equal %w[ cache_read_multi.active_support ], events.map(&:name)
     assert_equal :fetch_multi, events[0].payload[:super_operation]
+    assert_equal ["a", "b"], events[0].payload[:key]
     assert_equal ["b"], events[0].payload[:hits]
+    assert_equal @cache.class.name, events[0].payload[:store]
+  end
+
+  def test_instrumentation_empty_fetch_multi
+    events = with_instrumentation "read_multi" do
+      @cache.fetch_multi() { |key| key * 2 }
+    end
+
+    assert_equal %w[ cache_read_multi.active_support ], events.map(&:name)
+    assert_equal :fetch_multi, events[0].payload[:super_operation]
+    assert_equal [], events[0].payload[:key]
+    assert_equal [], events[0].payload[:hits]
+    assert_equal @cache.class.name, events[0].payload[:store]
   end
 
   def test_read_multi_instrumentation
     @cache.write("b", "bb")
 
     events = with_instrumentation "read_multi" do
-      @cache.read_multi("a", "b") { |key| key * 2 }
+      @cache.read_multi("a", "b")
     end
 
     assert_equal %w[ cache_read_multi.active_support ], events.map(&:name)
+    assert_equal ["a", "b"], events[0].payload[:key]
     assert_equal ["b"], events[0].payload[:hits]
+    assert_equal @cache.class.name, events[0].payload[:store]
+  end
+
+  def test_empty_read_multi_instrumentation
+    events = with_instrumentation "read_multi" do
+      @cache.read_multi()
+    end
+
+    assert_equal %w[ cache_read_multi.active_support ], events.map(&:name)
+    assert_equal [], events[0].payload[:key]
+    assert_equal [], events[0].payload[:hits]
+    assert_equal @cache.class.name, events[0].payload[:store]
   end
 
   private
