@@ -1,6 +1,23 @@
 # frozen_string_literal: true
 
 module LocalCacheBehavior
+  def test_instrumentation_with_local_cache
+    events = with_instrumentation "write" do
+      @cache.write("foo", "bar")
+    end
+    assert_equal @cache.class.name, events[0].payload[:store]
+
+    events = with_instrumentation "read" do
+      @cache.with_local_cache do
+        @cache.read("foo")
+        @cache.read("foo")
+      end
+    end
+
+    expected = [@cache.class.name, "ActiveSupport::Cache::Strategy::LocalCache::LocalStore"]
+    assert_equal expected, events.map { |p| p.payload[:store] }
+  end
+
   def test_local_writes_are_persistent_on_the_remote_cache
     retval = @cache.with_local_cache do
       @cache.write("foo", "bar")
@@ -46,6 +63,15 @@ module LocalCacheBehavior
     end
   end
 
+  def test_local_cache_of_read_returns_a_copy_of_the_entry
+    @cache.with_local_cache do
+      @cache.write(:foo, type: "bar")
+      value = @cache.read(:foo)
+      assert_equal("bar", value.delete(:type))
+      assert_equal({ type: "bar" }, @cache.read(:foo))
+    end
+  end
+
   def test_local_cache_of_read
     @cache.write("foo", "bar")
     @cache.with_local_cache do
@@ -93,6 +119,24 @@ module LocalCacheBehavior
     end
   end
 
+  def test_local_cache_of_delete_matched
+    begin
+      @cache.delete_matched("*")
+    rescue NotImplementedError
+      skip
+    end
+
+    @cache.with_local_cache do
+      @cache.write("foo", "bar")
+      @cache.write("fop", "bar")
+      @cache.write("bar", "foo")
+      @cache.delete_matched("fo*")
+      assert_not @cache.exist?("foo")
+      assert_not @cache.exist?("fop")
+      assert_equal "foo", @cache.read("bar")
+    end
+  end
+
   def test_local_cache_of_exist
     @cache.with_local_cache do
       @cache.write("foo", "bar")
@@ -141,6 +185,25 @@ module LocalCacheBehavior
     end
   end
 
+  def test_initial_object_mutation_after_write
+    @cache.with_local_cache do
+      initial = +"bar"
+      @cache.write("foo", initial)
+      initial << "baz"
+      assert_equal "bar", @cache.read("foo")
+    end
+  end
+
+  def test_initial_object_mutation_after_fetch
+    @cache.with_local_cache do
+      initial = +"bar"
+      @cache.fetch("foo") { initial }
+      initial << "baz"
+      assert_equal "bar", @cache.read("foo")
+      assert_equal "bar", @cache.fetch("foo")
+    end
+  end
+
   def test_middleware
     app = lambda { |env|
       result = @cache.write("foo", "bar")
