@@ -1,9 +1,9 @@
 # frozen_string_literal: true
 
-require "abstract_unit"
+require_relative "abstract_unit"
 require "pp"
 require "active_support/dependencies"
-require "dependencies_test_helpers"
+require_relative "dependencies_test_helpers"
 
 module ModuleWithMissing
   mattr_accessor :missing_count
@@ -22,11 +22,13 @@ class DependenciesTest < ActiveSupport::TestCase
 
   setup do
     @loaded_features_copy = $LOADED_FEATURES.dup
+    $LOAD_PATH << "test"
   end
 
   teardown do
     ActiveSupport::Dependencies.clear
     $LOADED_FEATURES.replace(@loaded_features_copy)
+    $LOAD_PATH.pop
   end
 
   def test_depend_on_path
@@ -40,6 +42,13 @@ class DependenciesTest < ActiveSupport::TestCase
     assert_equal expected.path, e.path
   end
 
+  def test_depend_on_message
+    e = assert_raises(LoadError) do
+      ActiveSupport::Dependencies.depend_on "omgwtfbbq"
+    end
+    assert_equal "No such file to load -- omgwtfbbq.rb", e.message
+  end
+
   def test_require_dependency_accepts_an_object_which_implements_to_path
     o = Object.new
     def o.to_path; "dependencies/service_one"; end
