@@ -1,6 +1,6 @@
 # frozen_string_literal: true
 
-require "abstract_unit"
+require_relative "abstract_unit"
 require "active_support/core_ext/string/inflections"
 require "yaml"
 
@@ -88,6 +88,7 @@ class SafeBufferTest < ActiveSupport::TestCase
     next: nil,
     reverse: nil,
     rstrip: nil,
+    scrub: nil,
     slice: "foo",
     squeeze: nil,
     strip: nil,
@@ -101,13 +102,13 @@ class SafeBufferTest < ActiveSupport::TestCase
   }.each do |unsafe_method, dummy_args|
     test "Should not return safe buffer from #{unsafe_method}" do
       skip unless String.method_defined?(unsafe_method)
-      altered_buffer = @buffer.send(unsafe_method, *dummy_args)
+      altered_buffer = @buffer.public_send(unsafe_method, *dummy_args)
       assert_not_predicate altered_buffer, :html_safe?
     end
 
     test "Should not return safe buffer from #{unsafe_method}!" do
       skip unless String.method_defined?("#{unsafe_method}!")
-      @buffer.send("#{unsafe_method}!", *dummy_args)
+      @buffer.public_send("#{unsafe_method}!", *dummy_args)
       assert_not_predicate @buffer, :html_safe?
     end
   end
