@@ -9,7 +9,6 @@ require "action_controller"
 require "action_view"
 
 require "rails_guides/markdown"
-require "rails_guides/indexer"
 require "rails_guides/helpers"
 require "rails_guides/levenshtein"
 
@@ -130,7 +129,7 @@ module RailsGuides
         if guide.end_with?(".md")
           guide.sub(/md\z/, "html")
         else
-          guide.sub(/\.erb\z/, "")
+          guide.delete_suffix(".erb")
         end
       end
 
@@ -163,7 +162,7 @@ module RailsGuides
 
           # Generate the special pages like the home.
           # Passing a template handler in the template name is deprecated. So pass the file name without the extension.
-          result = view.render(layout: layout, formats: [$1.to_sym], file: $`)
+          result = view.render(layout: layout, formats: [$1.to_sym], template: $`)
         else
           body = File.read("#{@source_dir}/#{guide}")
           result = RailsGuides::Markdown.new(
