@@ -33,7 +33,7 @@ module ApplicationTests
           assert_match(/Created database/, output)
           assert File.exist?(expected_database)
           yield if block_given?
-          assert_equal expected_database, ActiveRecord::Base.connection_config[:database] if environment_loaded
+          assert_equal expected_database, ActiveRecord::Base.connection_db_config.database if environment_loaded
           output = rails("db:drop")
           assert_match(/Dropped database/, output)
           assert_not File.exist?(expected_database)
@@ -51,7 +51,8 @@ module ApplicationTests
 
       test "db:create and db:drop without database URL" do
         require "#{app_path}/config/environment"
-        db_create_and_drop ActiveRecord::Base.configurations[Rails.env]["database"]
+        db_config = ActiveRecord::Base.configurations.configs_for(env_name: Rails.env, name: "primary")
+        db_create_and_drop db_config.database
       end
 
       test "db:create and db:drop with database URL" do
@@ -60,6 +61,28 @@ module ApplicationTests
         db_create_and_drop database_url_db_name
       end
 
+      test "db:create and db:drop with database URL don't use YAML DBs" do
+        require "#{app_path}/config/environment"
+        set_database_url
+
+        File.write("#{app_path}/config/database.yml", <<~YAML)
+          test:
+            adapter: sqlite3
+            database: db/test.sqlite3
+
+          development:
+            adapter: sqlite3
+            database: db/development.sqlite3
+        YAML
+
+        with_rails_env "development" do
+          db_create_and_drop database_url_db_name do
+            assert_not File.exist?("#{app_path}/db/test.sqlite3")
+            assert_not File.exist?("#{app_path}/db/development.sqlite3")
+          end
+        end
+      end
+
       test "db:create and db:drop respect environment setting" do
         app_file "config/database.yml", <<-YAML
           <% 1 %>
@@ -150,7 +173,7 @@ module ApplicationTests
         db_create_and_drop("db/development.sqlite3", environment_loaded: false)
       end
 
-      test "db:create and db:drop dont raise errors when loading YAML with FIXME ERB" do
+      test "db:create and db:drop dont raise errors when loading YAML with single-line ERB" do
         app_file "config/database.yml", <<-YAML
           development:
             <%= Rails.application.config.database ? 'database: db/development.sqlite3' : 'database: db/development.sqlite3' %>
@@ -310,7 +333,8 @@ module ApplicationTests
 
       test "db:migrate and db:migrate:status without database_url" do
         require "#{app_path}/config/environment"
-        db_migrate_and_status ActiveRecord::Base.configurations[Rails.env]["database"]
+        db_config = ActiveRecord::Base.configurations.configs_for(env_name: Rails.env, name: "primary")
+        db_migrate_and_status db_config.database
       end
 
       test "db:migrate and db:migrate:status with database_url" do
@@ -321,10 +345,10 @@ module ApplicationTests
 
       def db_schema_dump
         Dir.chdir(app_path) do
-          rails "generate", "model", "book", "title:string"
+          args = ["generate", "model", "book", "title:string"]
+          rails args
           rails "db:migrate", "db:schema:dump"
-          schema_dump = File.read("db/schema.rb")
-          assert_match(/create_table \"books\"/, schema_dump)
+          assert_match(/create_table \"books\"/, File.read("db/schema.rb"))
         end
       end
 
@@ -377,20 +401,58 @@ module ApplicationTests
         db_schema_cache_dump("db/special_schema_cache.yml")
       end
 
+      test "db:schema:cache:dump custom env" do
+        @old_schema_cache_env = ENV["SCHEMA_CACHE"]
+        filename = "db/special_schema_cache.yml"
+        ENV["SCHEMA_CACHE"] = filename
+
+        db_schema_dump
+        db_schema_cache_dump(filename)
+      ensure
+        ENV["SCHEMA_CACHE"] = @old_schema_cache_env
+      end
+
+      test "db:schema:cache:dump first config wins" do
+        Dir.chdir(app_path) do
+          File.open("#{app_path}/config/database.yml", "w") do |f|
+            f.puts <<-YAML
+            default: &default
+              adapter: sqlite3
+              pool: 5
+              timeout: 5000
+              variables:
+                statement_timeout: 1000
+            development:
+              some_entry:
+                <<: *default
+                database: db/development.sqlite3
+              another_entry:
+                <<: *default
+                database: db/another_entry_development.sqlite3
+                migrations_paths: db/another_entry_migrate
+            YAML
+          end
+        end
+
+        db_schema_dump
+        db_schema_cache_dump
+      end
+
       def db_fixtures_load(expected_database)
         Dir.chdir(app_path) do
           rails "generate", "model", "book", "title:string"
           reload
           rails "db:migrate", "db:fixtures:load"
 
-          assert_match expected_database, ActiveRecord::Base.connection_config[:database]
+          assert_match expected_database, ActiveRecord::Base.connection_db_config.database
           assert_equal 2, Book.count
         end
       end
 
       test "db:fixtures:load without database_url" do
         require "#{app_path}/config/environment"
-        db_fixtures_load ActiveRecord::Base.configurations[Rails.env]["database"]
+        db_config = ActiveRecord::Base.configurations.configs_for(env_name: Rails.env, name: "primary")
+        db_fixtures_load db_config.database
       end
 
       test "db:fixtures:load with database_url" do
@@ -416,33 +478,47 @@ module ApplicationTests
           structure_dump = File.read("db/structure.sql")
           assert_match(/CREATE TABLE (?:IF NOT EXISTS )?\"books\"/, structure_dump)
           rails "environment", "db:drop", "db:structure:load"
-          assert_match expected_database, ActiveRecord::Base.connection_config[:database]
+          assert_match expected_database, ActiveRecord::Base.connection_db_config.database
           require "#{app_path}/app/models/book"
           # if structure is not loaded correctly, exception would be raised
           assert_equal 0, Book.count
         end
       end
 
+      ["dump", "load"].each do |command|
+        test "db:structure:#{command} is deprecated" do
+          add_to_config("config.active_support.deprecation = :stderr")
+          stderr_output = capture(:stderr) { rails("db:structure:#{command}", stderr: true, allow_failure: true) }
+          assert_match(/DEPRECATION WARNING: Using `bin\/rails db:structure:#{command}` is deprecated and will be removed in Rails 6.2/, stderr_output)
+        end
+      end
+
       test "db:structure:dump and db:structure:load without database_url" do
+        add_to_config "config.active_record.schema_format = :sql"
         require "#{app_path}/config/environment"
-        db_structure_dump_and_load ActiveRecord::Base.configurations[Rails.env]["database"]
+        db_config = ActiveRecord::Base.connection_db_config
+        db_structure_dump_and_load db_config.database
       end
 
       test "db:structure:dump and db:structure:load with database_url" do
+        add_to_config "config.active_record.schema_format = :sql"
         require "#{app_path}/config/environment"
         set_database_url
         db_structure_dump_and_load database_url_db_name
       end
 
       test "db:structure:dump and db:structure:load set ar_internal_metadata" do
+        add_to_config "config.active_record.schema_format = :sql"
         require "#{app_path}/config/environment"
-        db_structure_dump_and_load ActiveRecord::Base.configurations[Rails.env]["database"]
+        db_config = ActiveRecord::Base.connection_db_config
+        db_structure_dump_and_load db_config.database
 
         assert_equal "test", rails("runner", "-e", "test", "puts ActiveRecord::InternalMetadata[:environment]").strip
         assert_equal "development", rails("runner", "puts ActiveRecord::InternalMetadata[:environment]").strip
       end
 
       test "db:structure:dump does not dump schema information when no migrations are used" do
+        add_to_config "config.active_record.schema_format = :sql"
         # create table without migrations
         rails "runner", "ActiveRecord::Base.connection.create_table(:posts) {|t| t.string :title }"
 
@@ -467,6 +543,7 @@ module ApplicationTests
         rails "db:schema:load"
         assert_equal '["posts", "comments", "schema_migrations", "ar_internal_metadata"]', list_tables[]
 
+        add_to_config "config.active_record.schema_format = :sql"
         app_file "db/structure.sql", <<-SQL
           CREATE TABLE "users" ("id" INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL, "name" varchar(255));
         SQL
@@ -503,7 +580,7 @@ module ApplicationTests
 
       test "db:schema:load fails if schema.rb doesn't exist yet" do
         stderr_output = capture(:stderr) { rails("db:schema:load", stderr: true, allow_failure: true) }
-        assert_match(/Run `rails db:migrate` to create it/, stderr_output)
+        assert_match(/Run `bin\/rails db:migrate` to create it/, stderr_output)
       end
 
       def db_test_load_structure
@@ -515,16 +592,23 @@ module ApplicationTests
           require "#{app_path}/app/models/book"
           # if structure is not loaded correctly, exception would be raised
           assert_equal 0, Book.count
-          assert_match ActiveRecord::Base.configurations["test"]["database"],
-            ActiveRecord::Base.connection_config[:database]
+          db_config = ActiveRecord::Base.configurations.configs_for(env_name: "test", name: "primary")
+          assert_match db_config.database, ActiveRecord::Base.connection_db_config.database
         end
       end
 
       test "db:test:load_structure without database_url" do
+        add_to_config "config.active_record.schema_format = :sql"
         require "#{app_path}/config/environment"
         db_test_load_structure
       end
 
+      test "db:test:load_structure is deprecated" do
+        add_to_config("config.active_support.deprecation = :stderr")
+        stderr_output = capture(:stderr) { rails("db:test:load_structure", stderr: true, allow_failure: true) }
+        assert_match(/DEPRECATION WARNING: Using `bin\/rails db:test:load_structure` is deprecated and will be removed in Rails 6.2/, stderr_output)
+      end
+
       test "db:setup loads schema and seeds database" do
         @old_rails_env = ENV["RAILS_ENV"]
         @old_rack_env = ENV["RACK_ENV"]
@@ -540,7 +624,7 @@ module ApplicationTests
         RUBY
 
         app_file "db/seeds.rb", <<-RUBY
-          puts ActiveRecord::Base.connection_config[:database]
+          puts ActiveRecord::Base.connection_db_config.database
         RUBY
 
         database_path = rails("db:setup")
@@ -671,6 +755,20 @@ module ApplicationTests
         end
       end
 
+      test "db:prepare setup the database even if schema does not exist" do
+        Dir.chdir(app_path) do
+          use_postgresql(multi_db: true) # bug doesn't exist with sqlite3
+          output = rails("db:drop")
+          assert_match(/Dropped database/, output)
+
+          rails "generate", "model", "recipe", "title:string"
+          output = rails("db:prepare")
+          assert_match(/CreateRecipes: migrated/, output)
+        end
+      ensure
+        rails "db:drop" rescue nil
+      end
+
       test "db:prepare does not touch schema when dumping is disabled" do
         Dir.chdir(app_path) do
           rails "generate", "model", "book", "title:string"
