@@ -35,4 +35,14 @@ class JobGeneratorTest < Rails::Generators::TestCase
       assert_match(/class ApplicationJob < ActiveJob::Base/, job)
     end
   end
+
+  def test_job_suffix_is_not_duplicated
+    run_generator ["notifier_job"]
+
+    assert_no_file "app/jobs/notifier_job_job.rb"
+    assert_file "app/jobs/notifier_job.rb"
+
+    assert_no_file "test/jobs/notifier_job_job_test.rb"
+    assert_file "test/jobs/notifier_job_test.rb"
+  end
 end
