@@ -7,7 +7,11 @@ class ResourceGeneratorTest < Rails::Generators::TestCase
   include GeneratorsTestHelper
   arguments %w(account)
 
-  setup :copy_routes
+  def setup
+    super
+    copy_routes
+    Rails::Generators::ModelHelpers.skip_warn = false
+  end
 
   def test_help_with_inherited_options
     content = run_generator ["--help"]
@@ -61,7 +65,7 @@ class ResourceGeneratorTest < Rails::Generators::TestCase
   end
 
   def test_plural_names_are_singularized
-    content = run_generator ["accounts".freeze]
+    content = run_generator ["accounts"]
     assert_file "app/models/account.rb", /class Account < ApplicationRecord/
     assert_file "test/models/account_test.rb", /class AccountTest/
     assert_match(/\[WARNING\] The model name 'accounts' was recognized as a plural, using the singular 'account' instead\. Override with --force-plural or setup custom inflection rules for this noun before running the generator\./, content)
@@ -75,7 +79,7 @@ class ResourceGeneratorTest < Rails::Generators::TestCase
   end
 
   def test_mass_nouns_do_not_throw_warnings
-    content = run_generator ["sheep".freeze]
+    content = run_generator ["sheep"]
     assert_no_match(/\[WARNING\]/, content)
   end
 
